//
// $Id: alu_avr.cc,v 1.9 2003/08/01 16:43:59 nj Exp $
//
#include "alu_avr.h"
#include "constants.h"

//
// Set various control signals
//
// sensitive << idc_add << idc_adc << idc_adiw << idc_sub << idc_subi;
// sensitive << idc_sbc << idc_sbci << idc_sbiw << adiw_st << sbiw_st;
// sensitive << idc_cp << idc_cpc << idc_cpi << idc_cpse << idc_ror;
// sensitive << alu_c_flag_in << alu_z_flag_in << alu_z_flag_out_int
void alu_avr::addsubcontrol()
{
  // Do any of these operations?
  // If any of these instructions, subtract numbers!
  adder_nadd_sub = idc_sub | idc_subi | idc_sbc | idc_sbci | /*sbiw_st*/idc_sbiw | idc_cp | idc_cpc | idc_cpi | idc_cpse;

  // Flags
  alu_c_flag_in_int = alu_c_flag_in & (idc_adc | adiw_st | idc_sbc | idc_sbci | sbiw_st | idc_cpc | idc_ror);

  alu_z_flag_out = (alu_z_flag_out_int & ~(adiw_st | sbiw_st | idc_cpc | idc_sbc | idc_sbci)) | (alu_z_flag_in & alu_z_flag_out_int & (adiw_st | sbiw_st)) | (alu_z_flag_in & alu_z_flag_out_int & (idc_cpc | idc_sbc | idc_sbci));
}

//
// Set outputs based in signals
//
// sensitive << alu_n_flag_out_int << alu_v_flag_out_int << alu_c_flag_out_int << alu_data_out_int
void alu_avr::signalsout()
{
  alu_n_flag_out = alu_n_flag_out_int;
  alu_v_flag_out = alu_v_flag_out_int;
  alu_c_flag_out = alu_c_flag_out_int;
  alu_data_out   = alu_data_out_int;
}

//
// Inputs to the ALU functions
//
// sensitive << alu_data_d_in << alu_data_r_in << adder_d_in << adder_r_in << adder_carry
// sensitive << alu_c_flag_in_int << adder_nadd_sub
void alu_avr::aluinputs()
{
  // Variables for tmp signal values
  sc_lv<9> v_adder_out;
  sc_lv<9> v_adder_d_in  = adder_d_in.read();
  sc_lv<9> v_adder_r_in  = adder_r_in.read();
  sc_lv<9> v_adder_carry = adder_carry.read();
  sc_logic v_adder_nadd_sub = adder_nadd_sub;

  // Full adder, bit by bit
  v_adder_out[0] = v_adder_d_in[0] ^ v_adder_r_in[0] ^ alu_c_flag_in_int;
  v_adder_carry[0] = ((v_adder_d_in[0] ^ v_adder_nadd_sub) & v_adder_r_in[0]) | (((v_adder_d_in[0] ^ v_adder_nadd_sub) | v_adder_r_in[0]) & alu_c_flag_in_int);

  ADD_SUMMATOR(1);
  ADD_SUMMATOR(2);
  ADD_SUMMATOR(3);
  ADD_SUMMATOR(4);
  ADD_SUMMATOR(5);
  ADD_SUMMATOR(6);
  ADD_SUMMATOR(7);
  ADD_SUMMATOR(8);

  adder_out = v_adder_out;
  adder_carry = v_adder_carry;

  // Flags from adder instruction
  adder_v_flag_out = (((v_adder_d_in[7] & v_adder_r_in[7] & ~v_adder_out[7]) | 
		       (~v_adder_d_in[7] & ~v_adder_r_in[7] & v_adder_out[7])) & ~v_adder_nadd_sub) | 
    (((v_adder_d_in[7] & ~v_adder_r_in[7] & ~v_adder_out[7]) |
      (~v_adder_d_in[7] & v_adder_r_in[7] & v_adder_out[7])) & v_adder_nadd_sub);
}

//
// Neg operation
//
// sensitive << alu_data_d_in << neg_op_carry;
void alu_avr::neg_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<9> v_neg_op_carry = neg_op_carry;
  sc_lv<9> v_neg_op_out;

  v_neg_op_out[0] = ~v_alu_data_d_in[0] ^ 1;
  v_neg_op_carry[0] = ~v_alu_data_d_in[0] & 1;

  NEG_SUMMATOR(1);
  NEG_SUMMATOR(2);
  NEG_SUMMATOR(3);
  NEG_SUMMATOR(4);
  NEG_SUMMATOR(5);
  NEG_SUMMATOR(6);
  NEG_SUMMATOR(7);

  v_neg_op_out[8] = v_neg_op_carry[7] ^ 1;
  v_neg_op_carry[8] = v_neg_op_carry[7];

  neg_op_out = v_neg_op_out;
  neg_op_carry = v_neg_op_carry;
}

//
// Inc, dec operations
//
// sensitive << alu_data_d_in << incdec_op_carry << idc_dec << incdec_op_out;
void alu_avr::incdec_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<8> v_incdec_op_out = incdec_op_out;
  sc_lv<8> v_incdec_op_carry = incdec_op_carry;

  v_incdec_op_out[0] = v_alu_data_d_in[0] ^ 1;
  v_incdec_op_carry[0] = v_alu_data_d_in[0] ^ idc_dec;

  INCDEC_GENERATE(1);
  INCDEC_GENERATE(2);
  INCDEC_GENERATE(3);
  INCDEC_GENERATE(4);
  INCDEC_GENERATE(5);
  INCDEC_GENERATE(6);
  INCDEC_GENERATE(7);

  incdec_op_out = v_incdec_op_out;
  incdec_op_carry = v_incdec_op_carry;
}

//
// Complement operation
//
// sensitive << alu_data_d_in;
void alu_avr::com_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();  
  com_op_out = ~v_alu_data_d_in;
}

//
// Binary operations with 2 operands
//
// sensitive << alu_data_d_in << alu_data_r_in;
void alu_avr::bool_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<8> v_alu_data_r_in = alu_data_r_in.read();

  and_op_out = v_alu_data_d_in & v_alu_data_r_in;
  or_op_out = v_alu_data_d_in | v_alu_data_r_in;
  eor_op_out = v_alu_data_d_in ^ v_alu_data_r_in;
}

//
// Shift operations
//
// sensitive << idc_ror << alu_c_flag_in_int << idc_asr << alu_data_d_in;
void alu_avr::shift_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<8> v_right_shift_out;

  v_right_shift_out[7] = (idc_ror & alu_c_flag_in_int) | (idc_asr & v_alu_data_d_in[7]);
  v_right_shift_out[6] = v_alu_data_d_in[7];
  v_right_shift_out[5] = v_alu_data_d_in[6];
  v_right_shift_out[4] = v_alu_data_d_in[5];
  v_right_shift_out[3] = v_alu_data_d_in[4];
  v_right_shift_out[2] = v_alu_data_d_in[3];
  v_right_shift_out[1] = v_alu_data_d_in[2];
  v_right_shift_out[0] = v_alu_data_d_in[1];

  right_shift_out = v_right_shift_out;
}

//
// SWAP operation, swaps nibbles
//
// sensitive << alu_data_d_in;
void alu_avr::swap_operation()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<8> v_swap_out;

  v_swap_out[7] = v_alu_data_d_in[3];
  v_swap_out[6] = v_alu_data_d_in[2];
  v_swap_out[5] = v_alu_data_d_in[1];
  v_swap_out[4] = v_alu_data_d_in[0];

  v_swap_out[3] = v_alu_data_d_in[7];
  v_swap_out[2] = v_alu_data_d_in[6];
  v_swap_out[1] = v_alu_data_d_in[5];
  v_swap_out[0] = v_alu_data_d_in[4];

  swap_out = v_swap_out;
}

//
// ALU output multiplexor
//
// sensitive << ALT!!
void alu_avr::output_mux()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<9> v_adder_out = adder_out;
  sc_lv<9> v_adder_carry = adder_carry;
  sc_lv<8> v_incdec_op_out = incdec_op_out;
  sc_lv<8> v_incdec_op_carry = incdec_op_carry;
  sc_lv<9> v_neg_op_out = neg_op_out;
  sc_lv<9> v_neg_op_carry = neg_op_carry;
  sc_lv<8> v_com_op_out = com_op_out;
  sc_lv<8> v_and_op_out = and_op_out;
  sc_lv<8> v_or_op_out = or_op_out;
  sc_lv<8> v_eor_op_out = eor_op_out;
  sc_lv<8> v_right_shift_out = right_shift_out;
  sc_lv<8> v_swap_out = swap_out;
  sc_lv<8> v_alu_data_out_int = alu_data_out_int;

  sc_logic v_alu_z_flag_out_int = alu_z_flag_out_int;

  // Set output
  ALUMUX_GENERATE(0);
  ALUMUX_GENERATE(1);
  ALUMUX_GENERATE(2);
  ALUMUX_GENERATE(3);
  ALUMUX_GENERATE(4);
  ALUMUX_GENERATE(5);
  ALUMUX_GENERATE(6);
  ALUMUX_GENERATE(7);

  // Set all flags as well
  alu_h_flag_out = (v_adder_carry[3] & (idc_add | idc_adc | idc_sub | idc_subi | idc_sbc | idc_sbci | idc_cp | idc_cpc | idc_cpi)) | (v_neg_op_carry[3] & idc_neg);

  alu_s_flag_out = alu_n_flag_out_int ^ alu_v_flag_out_int;

  alu_v_flag_out_int = (adder_v_flag_out & (idc_add | idc_adc | idc_sub | idc_subi | idc_sbc | idc_sbci | adiw_st | sbiw_st | idc_cp | idc_cpi | idc_cpc)) | ((v_alu_data_d_in[7] & v_neg_op_carry[6]) & idc_neg) | (~v_alu_data_d_in[7] & v_incdec_op_carry[6] & idc_inc) | (v_alu_data_d_in[7] & v_incdec_op_carry[6] & idc_dec) | ((alu_n_flag_out_int ^ alu_c_flag_out_int) & (idc_lsr | idc_ror | idc_asr));

  alu_n_flag_out_int = v_alu_data_out_int[7];

  if (v_alu_data_out_int == "00000000")
    alu_z_flag_out_int = SC_LOGIC_1;
  else
    alu_z_flag_out_int = SC_LOGIC_0;

  alu_c_flag_out_int = (v_adder_out[8] & (idc_add | idc_adc | (idc_adiw | adiw_st) | idc_sub | idc_subi | idc_sbc | idc_sbci | (idc_sbiw | sbiw_st) | idc_cp | idc_cpc | idc_cpi)) | 
    (~v_alu_z_flag_out_int & idc_neg) | 
    (v_alu_data_d_in[0] & (idc_lsr | idc_ror | idc_asr)) | idc_com;

  alu_data_out_int = v_alu_data_out_int;

  /*
  cout << "ALU output" << endl;
  cout << "alu_data_r_in = " << alu_data_r_in.read() << endl;
  cout << "alu_data_d_in = " << alu_data_d_in.read() << endl;
  cout << "alu_data_out_int = " << v_alu_data_out_int << endl; 
  */
}

// 
// Set adder_d_in and adder_r_in
// 
// sensitive << alu_data_d_in << alu_data_r_in
void alu_avr::set_adder_x_in()
{
  sc_lv<8> v_alu_data_d_in = alu_data_d_in.read();
  sc_lv<8> v_alu_data_r_in = alu_data_r_in.read();
  sc_lv<9> v_adder_d_in, v_adder_r_in;

  v_adder_d_in = ("0", v_alu_data_d_in);
  v_adder_r_in = ("0", v_alu_data_r_in);

  adder_d_in = v_adder_d_in;
  adder_r_in = v_adder_r_in;
}
