//
// $Id: alu_avr.h,v 1.10 2003/09/17 11:00:54 nj Exp $
//
#ifndef _ALU_AVR_H
#define _ALU_AVR_H
#include <systemc.h>
#include "config.h"

#define ADD_SUMMATOR(x) v_adder_out[x] = v_adder_d_in[x] ^ v_adder_r_in[x] ^ v_adder_carry[x-1]; \
  v_adder_carry[x] = ((v_adder_d_in[x] ^ v_adder_nadd_sub) & v_adder_r_in[x]) | \
(((v_adder_d_in[x] ^ v_adder_nadd_sub) | v_adder_r_in[x]) & v_adder_carry[x-1]);

#define NEG_SUMMATOR(x) v_neg_op_out[x] = ~v_alu_data_d_in[x] ^ v_neg_op_carry[x-1]; \
  v_neg_op_carry[x] = ~v_alu_data_d_in[x] & v_neg_op_carry[x-1];

#define INCDEC_GENERATE(x) v_incdec_op_out[x] = v_alu_data_d_in[x] ^ v_incdec_op_carry[x-1]; \
  v_incdec_op_carry[x] = (v_alu_data_d_in[x] ^ idc_dec) & v_incdec_op_carry[x-1];

#define ALUMUX_GENERATE(x) v_alu_data_out_int[x] = (v_adder_out[x] & (idc_add | idc_adc | idc_adiw | adiw_st | idc_sub | idc_subi | idc_sbc | idc_sbci | idc_sbiw | sbiw_st | idc_cpse | idc_cp | idc_cpc | idc_cpi)) | \
(v_neg_op_out[x] & idc_neg) | \
(v_incdec_op_out[x] & (idc_inc | idc_dec)) | \
(v_com_op_out[x] & idc_com) | \
(v_and_op_out[x] & (idc_and | idc_andi)) | \
(v_or_op_out[x] & (idc_or | idc_ori)) | \
(v_eor_op_out[x] & idc_eor) | \
(v_right_shift_out[x] & (idc_lsr | idc_ror | idc_asr)) | \
(v_swap_out[x] & idc_swap);

SC_MODULE(alu_avr)
{
  sc_in<sc_lv<8> > alu_data_r_in;
  sc_in<sc_lv<8> > alu_data_d_in;
  sc_in<sc_logic> alu_c_flag_in;
  sc_in<sc_logic> alu_z_flag_in;

  // Operation signal inputs
  sc_in<sc_logic> idc_add;
  sc_in<sc_logic> idc_adc;
  sc_in<sc_logic> idc_adiw;
  sc_in<sc_logic> idc_sub;
  sc_in<sc_logic> idc_subi;
  sc_in<sc_logic> idc_sbc;
  sc_in<sc_logic> idc_sbci;
  sc_in<sc_logic> idc_sbiw;

  sc_in<sc_logic> adiw_st;
  sc_in<sc_logic> sbiw_st;

  sc_in<sc_logic> idc_and;
  sc_in<sc_logic> idc_andi;
  sc_in<sc_logic> idc_or;
  sc_in<sc_logic> idc_ori;
  sc_in<sc_logic> idc_eor;
  sc_in<sc_logic> idc_com;
  sc_in<sc_logic> idc_neg;

  sc_in<sc_logic> idc_inc;
  sc_in<sc_logic> idc_dec;

  sc_in<sc_logic> idc_cp;
  sc_in<sc_logic> idc_cpc;
  sc_in<sc_logic> idc_cpi;
  sc_in<sc_logic> idc_cpse;
  sc_in<sc_logic> idc_lsr;
  sc_in<sc_logic> idc_ror;
  sc_in<sc_logic> idc_asr;
  sc_in<sc_logic> idc_swap;

  // Data output
  sc_out<sc_lv<8> > alu_data_out;
  
  // Flags output
  sc_out<sc_logic> alu_c_flag_out;
  sc_out<sc_logic> alu_z_flag_out;
  sc_out<sc_logic> alu_n_flag_out;
  sc_out<sc_logic> alu_v_flag_out;
  sc_out<sc_logic> alu_s_flag_out;
  sc_out<sc_logic> alu_h_flag_out;

  //
  // Internal signals
  //
  // *int signals are inputs to a mux
  sc_signal<sc_lv<8> > alu_data_out_int;

  // ALU flags
  sc_signal<sc_logic> alu_z_flag_out_int;
  sc_signal<sc_logic> alu_c_flag_in_int;

  sc_signal<sc_logic> alu_n_flag_out_int;
  sc_signal<sc_logic> alu_v_flag_out_int;
  sc_signal<sc_logic> alu_c_flag_out_int;

  // Adder signals
  sc_signal<sc_logic> adder_nadd_sub;
  sc_signal<sc_logic> adder_v_flag_out;

  sc_signal<sc_lv<9> > adder_carry;
  sc_signal<sc_lv<9> > adder_d_in;
  sc_signal<sc_lv<9> > adder_r_in;
  sc_signal<sc_lv<9> > adder_out;

  // Neg operator signals
  sc_signal<sc_lv<8> > neg_op_in;
  sc_signal<sc_lv<9> > neg_op_carry;
  sc_signal<sc_lv<9> > neg_op_out;

  // Inc, dec operator signals
  sc_signal<sc_lv<8> > incdec_op_in;
  sc_signal<sc_lv<8> > incdec_op_carry;
  sc_signal<sc_lv<8> > incdec_op_out;

  // Misc. signals
  sc_signal<sc_lv<8> > com_op_out;
  sc_signal<sc_lv<8> > and_op_out;
  sc_signal<sc_lv<8> > or_op_out;
  sc_signal<sc_lv<8> > eor_op_out;

  // Shift signals
  sc_signal<sc_lv<8> > right_shift_out;

  // Swap signals
  sc_signal<sc_lv<8> > swap_out;

  void addsubcontrol();
  void aluinputs();
  void signalsout();
  void neg_operation();
  void incdec_operation();
  void com_operation();
  void bool_operation();
  void shift_operation();
  void swap_operation();
  void set_adder_x_in();
  void output_mux();

  SC_CTOR(alu_avr)
  {
    SC_METHOD(addsubcontrol);
    sensitive << idc_add << idc_adc << idc_adiw << idc_sub << idc_subi;
    sensitive << idc_sbc << idc_sbci << idc_sbiw << adiw_st << sbiw_st;
    sensitive << idc_cp << idc_cpc << idc_cpi << idc_cpse << idc_ror;
    sensitive << alu_c_flag_in << alu_z_flag_in << alu_z_flag_out_int;

    SC_METHOD(aluinputs);
    sensitive << alu_data_d_in << alu_data_r_in << adder_d_in << adder_r_in << adder_carry;
    sensitive << alu_c_flag_in_int << adder_nadd_sub;

    SC_METHOD(signalsout);
    sensitive << alu_n_flag_out_int << alu_v_flag_out_int << alu_c_flag_out_int << alu_data_out_int;

    SC_METHOD(neg_operation);
    sensitive << alu_data_d_in << neg_op_carry;

    SC_METHOD(incdec_operation);
    sensitive << alu_data_d_in << incdec_op_carry << idc_dec << incdec_op_out;

    SC_METHOD(com_operation);
    sensitive << alu_data_d_in;

    SC_METHOD(bool_operation);
    sensitive << alu_data_d_in << alu_data_r_in;

    SC_METHOD(shift_operation);
    sensitive << idc_ror << alu_c_flag_in_int << idc_asr << alu_data_d_in;

    SC_METHOD(swap_operation);
    sensitive << alu_data_d_in;

    SC_METHOD(set_adder_x_in);
    sensitive << alu_data_d_in << alu_data_r_in;

    SC_METHOD(output_mux);
    sensitive << idc_add << idc_adc << idc_adiw << idc_sub << idc_subi;
    sensitive << idc_sbc << idc_sbci << idc_sbiw << adiw_st << sbiw_st;
    sensitive << idc_and << idc_andi << idc_or << idc_ori << idc_eor << idc_com << idc_neg;
    sensitive << idc_inc << idc_dec << idc_cp << idc_cpc << idc_cpi << idc_cpse << idc_ror;
    sensitive << idc_lsr << idc_ror << idc_asr << idc_swap;
    sensitive << adder_out << incdec_op_out << com_op_out << and_op_out << or_op_out << eor_op_out << right_shift_out << swap_out;
    sensitive << adder_carry << alu_z_flag_out_int << alu_c_flag_out_int << incdec_op_carry;
    sensitive << neg_op_carry << alu_data_d_in << adder_v_flag_out << alu_v_flag_out_int;
    sensitive << alu_data_out_int << neg_op_out << alu_n_flag_out_int;
  }

};

#endif
