//
// $Id: avr_core.h,v 1.15 2003/10/08 20:39:35 nj Exp $
//
#ifndef _PROM_H
#define _PROM_H
#include <systemc.h>
#include "config.h"

#include "alu_avr.h"
#include "pm_fetch_dec.h"
#include "reg_file.h"
#include "io_adr_dec.h"
#include "bit_processor.h"
#include "io_reg_file.h"

SC_MODULE(avr_core)
{
  // External interfaces to the core
  sc_in<bool> cp2;
  sc_in<bool> ireset;
  sc_in<sc_logic> cpuwait;

  // Program memory ports
  sc_out<sc_uint<16> > pc;
  sc_in<sc_lv<16> > inst;

  // I/O register ports
  sc_out<sc_uint<6> > adr;
  sc_out<sc_logic> iore;
  sc_out<sc_logic> iowe;

  // Data memory ports
  sc_out<sc_uint<16> > ramadr;
  sc_out<sc_logic> ramre;
  sc_out<sc_logic> ramwe;

  sc_in<sc_lv<8> > dbusin;
  sc_out<sc_lv<8> > dbusout;

  // Sleep controller ports
  sc_out<sc_logic> sleepi;
  sc_out<sc_logic> irqok;
  sc_out<sc_logic> globint;

  // Interrupt ports
  sc_in<sc_lv<23> > irqlines;
  sc_out<sc_logic> irqack;
  sc_out<sc_lv<5> > irqackad;

  // Non-standard ports for halt-state
  sc_in<sc_logic> pcwe;
  sc_out<sc_uint<16> > pc_in;
  sc_in<sc_uint<16> > pc_out;

  // register address when reading out registers...
  sc_in<sc_uint<5> > regadr;
  sc_in<sc_logic> regre;
  sc_in<sc_logic> regwe;
  sc_out<sc_lv<8> > reg_in;
  sc_in<sc_lv<8> > reg_out;


  // Signals for interconnecting modules
  sc_signal<sc_lv<8> > sg_dbusin;
  sc_signal<sc_lv<8> > sg_dbusout;
  sc_signal<sc_uint<6> > sg_adr;
  sc_signal<sc_logic> sg_iowe;
  sc_signal<sc_logic> sg_iore;
  sc_signal<sc_logic> sg_idc_adc;
  sc_signal<sc_logic> sg_idc_add;
  sc_signal<sc_logic> sg_idc_adiw;
  sc_signal<sc_logic> sg_idc_sub;
  sc_signal<sc_logic> sg_idc_subi;
  sc_signal<sc_logic> sg_idc_sbc;
  sc_signal<sc_logic> sg_idc_sbci;
  sc_signal<sc_logic> sg_idc_sbiw;
  sc_signal<sc_logic> sg_adiw_st;
  sc_signal<sc_logic> sg_sbiw_st;
  sc_signal<sc_logic> sg_idc_and;
  sc_signal<sc_logic> sg_idc_andi;
  sc_signal<sc_logic> sg_idc_or;
  sc_signal<sc_logic> sg_idc_ori;
  sc_signal<sc_logic> sg_idc_eor;
  sc_signal<sc_logic> sg_idc_com;
  sc_signal<sc_logic> sg_idc_neg;
  sc_signal<sc_logic> sg_idc_inc;
  sc_signal<sc_logic> sg_idc_dec;
  sc_signal<sc_logic> sg_idc_cp;
  sc_signal<sc_logic> sg_idc_cpc;
  sc_signal<sc_logic> sg_idc_cpi;
  sc_signal<sc_logic> sg_idc_cpse;
  sc_signal<sc_logic> sg_idc_lsr;
  sc_signal<sc_logic> sg_idc_ror;
  sc_signal<sc_logic> sg_idc_asr;
  sc_signal<sc_logic> sg_idc_swap;
  sc_signal<sc_logic> sg_idc_sbi;
  sc_signal<sc_logic> sg_sbi_st;
  sc_signal<sc_logic> sg_idc_cbi;
  sc_signal<sc_logic> sg_cbi_st;
  sc_signal<sc_logic> sg_idc_bld;
  sc_signal<sc_logic> sg_idc_bst;
  sc_signal<sc_logic> sg_idc_bset;
  sc_signal<sc_logic> sg_idc_bclr;
  sc_signal<sc_logic> sg_idc_sbic;
  sc_signal<sc_logic> sg_idc_sbis;
  sc_signal<sc_logic> sg_idc_sbrs;
  sc_signal<sc_logic> sg_idc_sbrc;
  sc_signal<sc_logic> sg_idc_brbs;
  sc_signal<sc_logic> sg_idc_brbc;
  sc_signal<sc_logic> sg_idc_reti;
  sc_signal<sc_lv<8> > sg_alu_data_r_in;
  sc_signal<sc_lv<8> > sg_alu_data_d_in;
  sc_signal<sc_lv<8> > sg_alu_data_out;
  sc_signal<sc_lv<8> > sg_reg_rd_in;
  sc_signal<sc_lv<8> > sg_reg_rd_out;
  sc_signal<sc_lv<8> > sg_reg_rr_out;
  sc_signal<sc_lv<5> > sg_reg_rd_adr;
  sc_signal<sc_lv<5> > sg_reg_rr_adr;
  sc_signal<sc_lv<16> > sg_reg_h_out;
  sc_signal<sc_lv<16> > sg_reg_z_out;
  sc_signal<sc_lv<3> > sg_reg_h_adr;
  sc_signal<sc_logic> sg_reg_rd_wr;
  sc_signal<sc_logic> sg_post_inc;
  sc_signal<sc_logic> sg_pre_dec;
  sc_signal<sc_logic> sg_reg_h_wr;
  sc_signal<sc_lv<8> > sg_sreg_fl_in;
  sc_signal<sc_lv<8> > sg_sreg_out;
  sc_signal<sc_lv<8> > sg_sreg_fl_wr_en;
  sc_signal<sc_lv<8> > sg_spl_out;
  sc_signal<sc_lv<8> > sg_sph_out;
  sc_signal<sc_lv<8> > sg_rampz_out;
  sc_signal<sc_logic> sg_sp_ndown_up;
  sc_signal<sc_logic> sg_sp_en;
  sc_signal<sc_lv<3> > sg_bit_num_r_io;
  sc_signal<sc_lv<3> > sg_branch;
  sc_signal<sc_lv<3> > sg_sreg_bit_num;
  sc_signal<sc_lv<8> > sg_bitpr_io_out;
  sc_signal<sc_lv<8> > sg_bit_pr_sreg_out;
  sc_signal<sc_lv<8> > sg_sreg_flags;
  sc_signal<sc_lv<8> > sg_bld_op_out;
  sc_signal<sc_lv<8> > sg_reg_file_rd_in;

  sc_signal<sc_logic> sg_bit_test_op_out;
  sc_signal<sc_logic> sg_alu_c_flag_out;
  sc_signal<sc_logic> sg_alu_z_flag_out;
  sc_signal<sc_logic> sg_alu_n_flag_out;
  sc_signal<sc_logic> sg_alu_v_flag_out;
  sc_signal<sc_logic> sg_alu_s_flag_out;
  sc_signal<sc_logic> sg_alu_h_flag_out;

  sc_signal<sc_logic> sg_sleepi;

  // systemC workaround signals
  sc_signal<sc_logic> sg_sreg_out_0;
  sc_signal<sc_logic> sg_sreg_out_1;

  void set_outputs();
  void set_sg_reg_signals();
  void set_sleep_signals();
  void clocktik()
    {
      cout << "############################################" << endl;
      cout << "#                   CLOCK                  #" << endl;
#ifndef SYNTHESIS
      cout << "#                    " << sc_simulation_time() / 20 << "                   #" << endl;
#endif
      cout << "############################################" << endl;
    };

  pm_fetch_dec *maincore;
  reg_file *gp_register_file;
  bit_processor *bit_proc;
  io_adr_dec *io_dec;
  io_reg_file *io_registers;
  alu_avr *alu;

  SC_CTOR(avr_core)
  {
    SC_METHOD(clocktik);
    sensitive_pos << cp2;
  
#ifdef WRITE_INIT
    cout << "INIT pm_fetch_dec (maincore)" << endl;
#endif
    maincore = new pm_fetch_dec("main_unit");
    maincore->clk(cp2);
    maincore->nrst(ireset);
    maincore->cpuwait(cpuwait);
    // Program memory ports
    maincore->pc(pc);
    maincore->inst(inst);
    // I/O Register ports
    maincore->adr(sg_adr);
    maincore->iore(sg_iore);
    maincore->iowe(sg_iowe);
    // Data memory ports
    maincore->ramadr(ramadr);
    maincore->ramre(ramre);
    maincore->ramwe(ramwe);
    maincore->dbusin(sg_dbusin);
    maincore->dbusout(sg_dbusout);
    // Interrupt ports
    maincore->irqlines(irqlines);
    maincore->irqack(irqack);
    maincore->irqackad(irqackad);
    // Interfaces to the alu
    maincore->alu_data_r_in(sg_alu_data_r_in);
    maincore->alu_data_d_in(sg_alu_data_d_in);
    // Operation signals inputs
    maincore->idc_add_out(sg_idc_add);
    maincore->idc_adc_out(sg_idc_adc);
    maincore->idc_adiw_out(sg_idc_adiw);
    maincore->idc_sub_out(sg_idc_sub);
    maincore->idc_subi_out(sg_idc_subi);
    maincore->idc_sbc_out(sg_idc_sbc);
    maincore->idc_sbci_out(sg_idc_sbci);
    maincore->idc_sbiw_out(sg_idc_sbiw);
    maincore->adiw_st_out(sg_adiw_st);
    maincore->sbiw_st_out(sg_sbiw_st);
    maincore->idc_and_out(sg_idc_and);
    maincore->idc_andi_out(sg_idc_andi);
    maincore->idc_or_out(sg_idc_or);
    maincore->idc_ori_out(sg_idc_ori);
    maincore->idc_eor_out(sg_idc_eor);
    maincore->idc_com_out(sg_idc_com);
    maincore->idc_neg_out(sg_idc_neg);
    maincore->idc_inc_out(sg_idc_inc);
    maincore->idc_dec_out(sg_idc_dec);
    maincore->idc_cp_out(sg_idc_cp);
    maincore->idc_cpc_out(sg_idc_cpc);
    maincore->idc_cpi_out(sg_idc_cpi);
    maincore->idc_cpse_out(sg_idc_cpse);
    maincore->idc_lsr_out(sg_idc_lsr);
    maincore->idc_ror_out(sg_idc_ror);
    maincore->idc_asr_out(sg_idc_asr);
    maincore->idc_swap_out(sg_idc_swap);

    // Alu data output
    maincore->alu_data_out(sg_alu_data_out);
    
    // Alu flags
    maincore->alu_c_flag_out(sg_alu_c_flag_out);
    maincore->alu_z_flag_out(sg_alu_z_flag_out);
    maincore->alu_n_flag_out(sg_alu_n_flag_out);
    maincore->alu_v_flag_out(sg_alu_v_flag_out);
    maincore->alu_s_flag_out(sg_alu_s_flag_out);
    maincore->alu_h_flag_out(sg_alu_h_flag_out);

    // Interfaces to the GPR
    maincore->reg_rd_in(sg_reg_rd_in);
    maincore->reg_rd_out(sg_reg_rd_out);
    maincore->reg_rd_adr(sg_reg_rd_adr);
    maincore->reg_rr_out(sg_reg_rr_out);
    maincore->reg_rr_adr(sg_reg_rr_adr);
    maincore->reg_rd_wr(sg_reg_rd_wr);
    maincore->post_inc(sg_post_inc);
    maincore->pre_dec(sg_pre_dec);
    maincore->reg_h_wr(sg_reg_h_wr);
    maincore->reg_h_out(sg_reg_h_out);
    maincore->reg_h_adr(sg_reg_h_adr);
    maincore->reg_z_out(sg_reg_z_out);
    
    // Interfaces to the input/output regfile
    maincore->sreg_fl_in(sg_sreg_fl_in);
    maincore->sreg_out(sg_sreg_out);
    maincore->sreg_fl_wr_en(sg_sreg_fl_wr_en);

    maincore->spl_out(sg_spl_out);
    maincore->sph_out(sg_sph_out);
    maincore->sp_ndown_up(sg_sp_ndown_up);
    maincore->sp_en(sg_sp_en);
    maincore->rampz_out(sg_rampz_out);
    
    // Interfaces to the bit_processor
    maincore->bit_num_r_io(sg_bit_num_r_io);
    maincore->bitpr_io_out(sg_bitpr_io_out);
    maincore->branch(sg_branch);
    maincore->bit_pr_sreg_out(sg_bit_pr_sreg_out);
    maincore->sreg_bit_num(sg_sreg_bit_num);
    maincore->bld_op_out(sg_bld_op_out);
    maincore->bit_test_op_out(sg_bit_test_op_out);

    // Instructions and states
    maincore->idc_sbi_out(sg_idc_sbi);
    maincore->sbi_st_out(sg_sbi_st);
    maincore->idc_cbi_out(sg_idc_cbi);
    maincore->cbi_st_out(sg_cbi_st);

    maincore->idc_bld_out(sg_idc_bld);
    maincore->idc_bst_out(sg_idc_bst);
    maincore->idc_bset_out(sg_idc_bset);
    maincore->idc_bclr_out(sg_idc_bclr);
    maincore->idc_sbic_out(sg_idc_sbic);
    maincore->idc_sbis_out(sg_idc_sbis);
    maincore->idc_sbrs_out(sg_idc_sbrs);
    maincore->idc_sbrc_out(sg_idc_sbrc);
    maincore->idc_brbs_out(sg_idc_brbs);
    maincore->idc_brbc_out(sg_idc_brbc);
    maincore->idc_reti_out(sg_idc_reti);

    maincore->sleepi(sg_sleepi);
    
    maincore->pcwe(pcwe);
    maincore->pc_in(pc_in);
    maincore->pc_out(pc_out);

    // Register file...
#ifdef WRITE_INIT
    cout << "INIT reg_file (gp_register_file)" << endl;
#endif
    gp_register_file = new reg_file("gp_regfile");
    gp_register_file->clk(cp2);       
    gp_register_file->nrst(ireset);
    gp_register_file->reg_rd_in(sg_reg_rd_in);
    gp_register_file->reg_rd_out(sg_reg_rd_out);
    gp_register_file->reg_rd_adr(sg_reg_rd_adr);
    gp_register_file->reg_rr_out(sg_reg_rr_out);
    gp_register_file->reg_rr_adr(sg_reg_rr_adr);
    gp_register_file->reg_rd_wr (sg_reg_rd_wr);
    
    gp_register_file->post_inc  (sg_post_inc);
    gp_register_file->pre_dec   (sg_pre_dec);
    gp_register_file->reg_h_wr  (sg_reg_h_wr);
    gp_register_file->reg_h_out (sg_reg_h_out);
    gp_register_file->reg_h_adr (sg_reg_h_adr);
    gp_register_file->reg_z_out (sg_reg_z_out);

    gp_register_file->regadr(regadr);
    gp_register_file->regre(regre);
    gp_register_file->regwe(regwe);
    gp_register_file->reg_in(reg_in);
    gp_register_file->reg_out(reg_out);

    // BIT processor
#ifdef WRITE_INIT
    cout << "INIT bit_processor (bit_proc)" << endl;
#endif
    bit_proc = new bit_processor("bit_processor");
    bit_proc->clk(cp2);                   
    bit_proc->nrst(ireset);                  
              
    bit_proc->bit_num_r_io(sg_bit_num_r_io);  
    bit_proc->dbusin(sg_dbusin);   
    bit_proc->bitpr_io_out(sg_bitpr_io_out);   
    
    bit_proc->sreg_out(sg_sreg_out);   
    bit_proc->branch(sg_branch);  
  
    bit_proc->bit_pr_sreg_out(sg_bit_pr_sreg_out);
    bit_proc->sreg_bit_num(sg_sreg_bit_num);
    
    bit_proc->bld_op_out(sg_bld_op_out);
    bit_proc->reg_rd_out(sg_reg_rd_out);
    
    bit_proc->bit_test_op_out(sg_bit_test_op_out);

    // OPERATION SIGNALS INPUTS
    // INSTRUCTUIONS AND STATES
    bit_proc->idc_sbi(sg_idc_sbi);
    bit_proc->sbi_st (sg_sbi_st);
    bit_proc->idc_cbi(sg_idc_cbi);
    bit_proc->cbi_st (sg_cbi_st);
    
    bit_proc->idc_bld(sg_idc_bld);
    bit_proc->idc_bst(sg_idc_bst);
    bit_proc->idc_bset(sg_idc_bset);
    bit_proc->idc_bclr(sg_idc_bclr);
    
    bit_proc->idc_sbic(sg_idc_sbic);       
    bit_proc->idc_sbis(sg_idc_sbis);       
    bit_proc->idc_sbrs(sg_idc_sbrs);        
    bit_proc->idc_sbrc(sg_idc_sbrc);        
              
    bit_proc->idc_brbs(sg_idc_brbs);        
    bit_proc->idc_brbc(sg_idc_brbc);        
    bit_proc->idc_reti(sg_idc_reti);


    // IO adr decoder
#ifdef WRITE_INIT
    cout << "INIT io_adr_dec (io_adr_decoder)" << endl;
#endif
    io_dec = new io_adr_dec("io_adr_decoder");
    io_dec->adr(sg_adr);
    io_dec->iore(sg_iore);
    io_dec->dbusin_int(sg_dbusin);   // LOCAL DATA BUS OUTPUT
    io_dec->dbusin_ext(dbusin);      // EXTERNAL DATA BUS INPUT
    
    io_dec->spl_out(sg_spl_out);
    io_dec->sph_out(sg_sph_out);
    io_dec->sreg_out(sg_sreg_out);
    io_dec->rampz_out(sg_rampz_out);
    

    // IO register
#ifdef WRITE_INIT
    cout << "INIT io_reg_file (io_registers)" << endl;
#endif
    io_registers = new io_reg_file("io_registers");
    io_registers->clk(cp2);
    io_registers->nrst(ireset);     
    
    io_registers->adr(sg_adr);       
    io_registers->iowe(sg_iowe);
    io_registers->dbusout(sg_dbusout);     
    
    io_registers->sreg_fl_in(sg_sreg_fl_in);
    io_registers->sreg_out(sg_sreg_out);
    
    io_registers->sreg_fl_wr_en(sg_sreg_fl_wr_en);
    
    io_registers->spl_out(sg_spl_out);    
    io_registers->sph_out(sg_sph_out);    
    io_registers->sp_ndown_up(sg_sp_ndown_up); 
    io_registers->sp_en(sg_sp_en);   
    io_registers->rampz_out(sg_rampz_out);


    // ALU
#ifdef WRITE_INIT
    cout << "INIT alu_avr (alu)" << endl;
#endif
    alu = new alu_avr("avr_alu");
    alu->alu_data_r_in(sg_alu_data_r_in);
    alu->alu_data_d_in(sg_alu_data_d_in);
    alu->alu_c_flag_in(sg_sreg_out_0);
    alu->alu_z_flag_in(sg_sreg_out_1);

    // OPERATION SIGNALS INPUTS
    alu->idc_add (sg_idc_add);
    alu->idc_adc (sg_idc_adc);      
    alu->idc_adiw(sg_idc_adiw);     
    alu->idc_sub (sg_idc_sub);     
    alu->idc_subi(sg_idc_subi);     
    alu->idc_sbc (sg_idc_sbc);     
    alu->idc_sbci(sg_idc_sbci);     
    alu->idc_sbiw(sg_idc_sbiw);     
    
    alu->adiw_st (sg_adiw_st);     
    alu->sbiw_st (sg_sbiw_st);     

    alu->idc_and (sg_idc_and);     
    alu->idc_andi(sg_idc_andi);     
    alu->idc_or  (sg_idc_or);     
    alu->idc_ori (sg_idc_ori);     
    alu->idc_eor (sg_idc_eor);     
    alu->idc_com (sg_idc_com);     
    alu->idc_neg (sg_idc_neg);     
    
    alu->idc_inc (sg_idc_inc);     
    alu->idc_dec (sg_idc_dec);     
    
    alu->idc_cp  (sg_idc_cp);     
    alu->idc_cpc (sg_idc_cpc);     
    alu->idc_cpi (sg_idc_cpi);    
    alu->idc_cpse(sg_idc_cpse);     
    
    alu->idc_lsr (sg_idc_lsr);     
    alu->idc_ror (sg_idc_ror);      
    alu->idc_asr (sg_idc_asr);      
    alu->idc_swap(sg_idc_swap);      
    
    // DATA OUTPUT
    alu->alu_data_out(sg_alu_data_out);  

    // FLAGS OUTPUT
    alu->alu_c_flag_out(sg_alu_c_flag_out);
    alu->alu_z_flag_out(sg_alu_z_flag_out);
    alu->alu_n_flag_out(sg_alu_n_flag_out);
    alu->alu_v_flag_out(sg_alu_v_flag_out);
    alu->alu_s_flag_out(sg_alu_s_flag_out);
    alu->alu_h_flag_out(sg_alu_h_flag_out);
 
    
    SC_METHOD(set_outputs);
    sensitive << sg_adr << sg_iowe << sg_iore << sg_dbusout;

    SC_METHOD(set_sg_reg_signals);
    sensitive << sg_sreg_out;

    SC_METHOD(set_sleep_signals);
    sensitive << irqlines << sg_sreg_out << sg_sleepi;
  }  
};

#endif
