//
// $Id: bit_processor.cc,v 1.7 2003/07/19 11:17:08 nj Exp $
//
#include "bit_processor.h"

//
// Set temp_in_data on rising clock edge
//
void bit_processor::sbi_cbi()
{
  /*
  cout << "bit_processor::sbi_cbi()" << endl;
  cout << "bit_processor got " << dbusin.read() << " from dbusin" << endl;
  */
  if (!nrst)
    temp_in_data = "00000000";
  else
    temp_in_data = dbusin.read();
}

//
// Output sreg t-flag
//
// sensitive << sreg_out
void bit_processor::output_sreg()
{
  sc_lv<8> v_sreg_out = sreg_out.read();

  sreg_t_flag = v_sreg_out[6];
}

//
// Set bitpr_io_out...
//
// sensitive << sbi_st << bit_num_decode << cbi_st << temp_in_data;
void bit_processor::sbi_cpi_logic()
{
  sc_lv<8> v_bitpr_io_out;
  sc_lv<8> v_bit_num_decode = bit_num_decode;
  sc_lv<8> v_temp_in_data = temp_in_data;

  SBI_CPI_GENERATE(0);
  SBI_CPI_GENERATE(1);
  SBI_CPI_GENERATE(2);
  SBI_CPI_GENERATE(3);
  SBI_CPI_GENERATE(4);
  SBI_CPI_GENERATE(5);
  SBI_CPI_GENERATE(6);
  SBI_CPI_GENERATE(7);

  bitpr_io_out = v_bitpr_io_out;
}

//
// BST, Load T bit of sreg from general purpose register
//
// sensitive << bit_num_r_io << reg_rd_out << sreg_t_temp;
void bit_processor::bst_part()
{
  sc_lv<8> v_bit_num_decode = bit_num_decode;
  sc_uint<3> i_bit_num_r_io = bit_num_r_io.read();
  sc_lv<8> v_sreg_t_temp = sreg_t_temp;
  sc_lv<8> v_reg_rd_out = reg_rd_out;

  v_bit_num_decode = "00000000";
  v_bit_num_decode[i_bit_num_r_io] = SC_LOGIC_1;

  if (v_bit_num_decode[0] == 1)
    v_sreg_t_temp[0] = v_reg_rd_out[0];
  else
    v_sreg_t_temp[0] = SC_LOGIC_0;

  SREG_GENERATE(1);
  SREG_GENERATE(2);
  SREG_GENERATE(3);
  SREG_GENERATE(4);
  SREG_GENERATE(5);
  SREG_GENERATE(6);
  SREG_GENERATE(7);

  bit_num_decode = v_bit_num_decode;
  sreg_t_temp = v_sreg_t_temp;
}

//
// BLD part
//
// sensitive << bld_op_out << sreg_t_flag << bit_num_r_io << reg_rd_out;
void bit_processor::bld_logic()
{
  sc_lv<8> v_bld_op_out = bld_op_out;
  sc_uint<3> i_bit_num_r_io = bit_num_r_io.read();
  
  v_bld_op_out = reg_rd_out.read();
  v_bld_op_out[i_bit_num_r_io] = sreg_t_flag;
}

//
// BCLR/BSET/BST/RETI logic
//
// sensitive << bit_pr_sreg_out_int << reg_rd_out << bit_pr_sreg_out << sreg_t_temp << idc_bset << idc_bclr << idc_reti << idc_bst
void bit_processor::bclr_bset_logic()
{
  sc_lv<8> v_bit_pr_sreg_out_int = bit_pr_sreg_out_int;
  sc_lv<8> v_bit_pr_sreg_out     = bit_pr_sreg_out;
  sc_lv<8> v_reg_rd_out = reg_rd_out;
  sc_lv<8> v_sreg_t_temp = sreg_t_temp;

  // Bit 0-6
  // #define BIT_PR_GENERATE(x) v_bit_pr_sreg_out_int[x] = (idc_bset & ~v_reg_rd_out[x]) | (~idc_bclr.read() & v_reg_rd_out[x]);
  // XXX - above line is original
  BIT_PR_GENERATE(0);
  BIT_PR_GENERATE(1);
  BIT_PR_GENERATE(2);
  BIT_PR_GENERATE(3);
  BIT_PR_GENERATE(4);
  BIT_PR_GENERATE(5);
  BIT_PR_GENERATE(6);

  // Bit 7, interrupt enable flag
  // XXX - Changed bset to bst
  v_bit_pr_sreg_out_int[7] = (idc_bst & ~v_reg_rd_out[7]) | (~idc_bclr.read() & v_reg_rd_out[7]) | idc_bset | idc_reti;


  if (idc_bset == SC_LOGIC_1) // XXX - was: idc_bst
    v_bit_pr_sreg_out = (v_bit_pr_sreg_out_int[7],v_sreg_t_temp[7],v_bit_pr_sreg_out_int.range(5,0));
  else
    v_bit_pr_sreg_out = bit_pr_sreg_out_int;

  bit_pr_sreg_out = v_bit_pr_sreg_out;
}

//
// SBIC/SBIS/SBRS/SBRC logic
//
// sensitive << dbusin << idc_sbis << idc_sbic << idc_sbrs << idc_sbrc << idc_brbs << idc_brbc
// sensitive << bit_num_decode << bit_test_in << reg_rd_out << branch_mux
void bit_processor::it_test_mux()
{
  sc_lv<8> v_bit_num_decode = bit_num_decode;
  sc_lv<8> v_bit_test_in = bit_test_in;
  sc_lv<8> v_branch_mux = branch_mux;
  sc_lv<8> v_bit_test_mux_out;

  if ((idc_sbis == SC_LOGIC_1) | (idc_sbic == SC_LOGIC_1))
    bit_test_in = dbusin;
  else
    bit_test_in = reg_rd_out;

  if (v_bit_num_decode[0] == SC_LOGIC_1)
    v_bit_test_mux_out[0] = v_bit_test_in[0];
  else
    v_bit_test_mux_out[0] = SC_LOGIC_0;

  BIT_TEST_GENERATE(1);
  BIT_TEST_GENERATE(2);
  BIT_TEST_GENERATE(3);
  BIT_TEST_GENERATE(4);
  BIT_TEST_GENERATE(5);
  BIT_TEST_GENERATE(6);
  BIT_TEST_GENERATE(7);

  bit_test_op_out = (v_bit_test_mux_out[7] & (idc_sbis | idc_sbrs)) | (~v_bit_test_mux_out[7] & (idc_sbic | idc_sbrc)) | (v_branch_mux[7] & idc_brbs) | (~v_branch_mux[7] & idc_brbc);

  bit_test_mux_out = v_bit_test_mux_out;
}

//
// BRBS/BRBC logic
//
// sensitive << branch << branch_mux << sreg_out;
void bit_processor::branch_decode_logic()
{
  sc_lv<8> v_branch_decode;
  sc_uint<3> i_branch = branch.read();
  sc_lv<8> v_branch_mux = branch_mux;
  sc_lv<8> v_sreg_out = sreg_out;

  v_branch_decode = "00000000";
  v_branch_decode[i_branch] = SC_LOGIC_1;

  if (v_branch_decode[0] == SC_LOGIC_1)
    v_branch_mux[0] = v_sreg_out[0];
  else
    v_branch_mux[0] = SC_LOGIC_0;

  BRANCH_MUX_GENERATE(1);
  BRANCH_MUX_GENERATE(2);
  BRANCH_MUX_GENERATE(3);
  BRANCH_MUX_GENERATE(4);
  BRANCH_MUX_GENERATE(5);
  BRANCH_MUX_GENERATE(6);  
  BRANCH_MUX_GENERATE(7);

  branch_mux = v_branch_mux;
  branch_decode = v_branch_decode;			      
}
