//
// $Id: bit_processor.h,v 1.5 2003/07/19 11:17:08 nj Exp $
//
#ifndef _BIT_PROCESSOR_H
#define _BIT_PROCESSOR_H
#include <systemc.h>
#include "config.h"
#include "constants.h"

#define SBI_CPI_GENERATE(x) if ((sbi_st == SC_LOGIC_1) & (v_bit_num_decode[x] == SC_LOGIC_1)) \
    v_bitpr_io_out[x] = SC_LOGIC_1; \
  else if ((cbi_st == SC_LOGIC_1) & (v_bit_num_decode[x] == 1)) \
    v_bitpr_io_out[x] = SC_LOGIC_0; \
  else \
    v_bitpr_io_out[x] = v_temp_in_data[x];

#define SREG_GENERATE(x) if (v_bit_num_decode[x] == 1) \
    v_sreg_t_temp[x] = v_reg_rd_out[x]; \
  else \
    v_sreg_t_temp[x] = v_sreg_t_temp[x-1];

#define BIT_PR_GENERATE(x) v_bit_pr_sreg_out_int[x] = (idc_bst & ~v_reg_rd_out[x]) | (~idc_bclr.read() & v_reg_rd_out[x]) | idc_bset;

#define BIT_TEST_GENERATE(x) if (v_bit_num_decode[x] == 1) \
    v_bit_test_mux_out[x] = v_bit_test_in[x]; \
  else \
    v_bit_test_mux_out[x] = v_bit_test_mux_out[x-1];

#define BRANCH_MUX_GENERATE(x) if (v_branch_decode[x] == 1) \
    v_branch_mux[x] = v_sreg_out[x]; \
  else \
    v_branch_mux[x] = v_branch_mux[x-1];

SC_MODULE(bit_processor)
{
  sc_in<bool> clk;
  sc_in<bool> nrst;

  sc_in<sc_lv<3> > bit_num_r_io;
  sc_in<sc_lv<8> > dbusin;
  sc_out<sc_lv<8> > bitpr_io_out;
  sc_in<sc_lv<8> > sreg_out;
  sc_in<sc_lv<3> > branch;
  sc_out<sc_lv<8> > bit_pr_sreg_out;
  sc_in<sc_lv<3> > sreg_bit_num;
  sc_out<sc_lv<8> > bld_op_out;
  sc_in<sc_lv<8> > reg_rd_out;
  sc_out<sc_logic> bit_test_op_out;

  sc_in<sc_logic> idc_sbi;
  sc_in<sc_logic> sbi_st;
  sc_in<sc_logic> idc_cbi;
  sc_in<sc_logic> cbi_st;
  sc_in<sc_logic> idc_bld;
  sc_in<sc_logic> idc_bst;
  sc_in<sc_logic> idc_bset;
  sc_in<sc_logic> idc_bclr;
  sc_in<sc_logic> idc_sbic;
  sc_in<sc_logic> idc_sbis;
  sc_in<sc_logic> idc_sbrs;
  sc_in<sc_logic> idc_sbrc;
  sc_in<sc_logic> idc_brbs;
  sc_in<sc_logic> idc_brbc;
  sc_in<sc_logic> idc_reti;

  sc_signal<sc_logic> sreg_t_flag;
  sc_signal<sc_lv<8> > temp_in_data;
  sc_signal<sc_lv<8> > sreg_t_temp;
  sc_signal<sc_lv<8> > bit_num_decode;
  sc_signal<sc_lv<8> > bit_pr_sreg_out_int;
  sc_signal<sc_lv<8> > bit_test_in;
  sc_signal<sc_lv<8> > bit_test_mux_out;
  sc_signal<sc_lv<8> > branch_decode;
  sc_signal<sc_lv<8> > branch_mux;

  void sbi_cbi();
  void output_sreg();
  void sbi_cpi_logic();
  void bst_part();
  void bld_logic();
  void bclr_bset_logic();
  void it_test_mux();
  void branch_decode_logic();

  SC_CTOR(bit_processor)
  {
    SC_METHOD(sbi_cbi);
    sensitive_pos << clk;

    SC_METHOD(output_sreg);
    sensitive << sreg_out;

    SC_METHOD(sbi_cpi_logic);
    sensitive << sbi_st << bit_num_decode << cbi_st << temp_in_data;
    
    SC_METHOD(bst_part);
    sensitive << bit_num_r_io << reg_rd_out << sreg_t_temp;

    SC_METHOD(bld_logic);
    sensitive << bld_op_out << sreg_t_flag << bit_num_r_io << reg_rd_out;

    SC_METHOD(bclr_bset_logic);
    sensitive << bit_pr_sreg_out_int << reg_rd_out << bit_pr_sreg_out << sreg_t_temp << idc_bset << idc_bclr << idc_reti << idc_bst;

    SC_METHOD(it_test_mux);
    sensitive << dbusin << idc_sbis << idc_sbic << idc_sbrs << idc_sbrc << idc_brbs << idc_brbc;
    sensitive << bit_num_decode << bit_test_in << reg_rd_out << branch_mux;

    SC_METHOD(branch_decode_logic);
    sensitive << branch << branch_mux << sreg_out;
  }
};

#endif
