//
// $Id: halt_cpu.h,v 1.3 2003/10/08 20:39:35 nj Exp $
//
#ifndef _HALT_CPU_H
#define _HALT_CPU_H
#include <systemc.h>
#include "config.h"

enum halt_states { idle = 1, start_state, copyregs, halted, copyback, end_state };

SC_MODULE(halt_cpu)
{
  // External interfaces to the core
  sc_in<bool> clk;
  sc_in<bool> reset;

  // tell avr_core to place nops while we read registers
  sc_out<sc_logic> cpuwait;

  // start the show?
  sc_in<bool> halt;

  // read/write program counter
  sc_out<sc_logic> pcwe;
  sc_in<sc_uint<16> > pc_in;
  sc_out<sc_uint<16> > pc_out;
  // register address when reading out registers...
  sc_out<sc_uint<5> > regadr;
  sc_out<sc_logic> regre;
  sc_out<sc_logic> regwe;
  sc_in<sc_lv<8> > reg_in;
  sc_out<sc_lv<8> > reg_out;

  sc_signal<sc_logic> cpuwait_i;

  // FSM
  sc_signal<unsigned int> state;
  // the current reg being saved...
  sc_signal<unsigned int> regno;

  // saved registers
  sc_signal<sc_lv<8> > saveregs[32];
  // save program counter
  sc_signal<sc_uint<16> > savepc;

  void execute();
  void execute_neg();
  void setcpuwait();

  SC_CTOR(halt_cpu)
  {

    SC_METHOD(execute);
    sensitive_pos << clk;

    SC_METHOD(execute_neg);
    sensitive_neg << clk;

    SC_METHOD(setcpuwait);
    sensitive << cpuwait_i << halt;

  }

};

#endif
