//
// $Id: io_reg_file.cc,v 1.7 2003/07/19 11:17:08 nj Exp $
//
#include "io_reg_file.h"

//
// Write Sreg
//
void io_reg_file::sreg_write()
{
  sc_lv<8> v_sreg_fl_wr_en = sreg_fl_wr_en;
  sc_lv<8> v_sreg_fl_in = sreg_fl_in;
  sc_lv<8> v_dbusout = dbusout;
  sc_lv<6> ladr = adr.read();

  // Output
  sc_lv<8> v_sreg = sreg.read();
  
  if (!nrst)
    {
      v_sreg = "00000000";
    }
  else
    {/*
      #define SREG_UNROLL(x) if ((v_sreg_fl_wr_en[x] == 1) || ((ladr == SREG_Address) && (iowe == 1))) \
  { \
    if (iowe == 1) \
      v_sreg[x] = v_dbusout[x]; \
    else \
      v_sreg[x] = v_sreg_fl_in[x]; \
  }
     */
      SREG_UNROLL(0);
      SREG_UNROLL(1);
      SREG_UNROLL(2);
      SREG_UNROLL(3);
      SREG_UNROLL(4);
      SREG_UNROLL(5);
      SREG_UNROLL(6);
      SREG_UNROLL(7);
    }

  sreg      = v_sreg;
  sreg_out  = v_sreg;

  /*
  cout << "io_reg_file::sreg_write()" << endl;
  cout << "sreg (ITHS.VNZC) = " << v_sreg << endl;
  cout << "dbusout = " << v_dbusout << endl;
  cout << "flag_enable = " << v_sreg_fl_wr_en << endl;
  cout << "sreg_fl_in = " << v_sreg_fl_in << endl;
  */
}

//
// Write SPL
//
void io_reg_file::spl_write()
{
  sc_lv<16> v_sp_res = sp_res;
  sc_lv<6> ladr = adr.read();

  if (!nrst)
    {  
      spl = "00000000";
    }
  else
    {
      if ((sp_en == 1) || ((ladr == SPL_Address) && (iowe == 1)))
      {
        if (iowe == 1)
	  spl = dbusout;
        else 
	  spl = v_sp_res.range(7,0);
      }
    }
}

//
// Write sph
//
void io_reg_file::sph_write()
{
  sc_lv<16> v_sp_res = sp_res;
  sc_lv<6>  ladr = adr.read();

  if (!nrst)
    {
      sph = "00000000";
    }
  else
    {
      if ((sp_en == 1) || ((ladr == SPH_Address) && (iowe == 1)))
	{
	  if (iowe == 1)
	    sph = dbusout;
	  else 
	    sph = v_sp_res.range(15,8);
	}
    }
}

//
// Write RAMPZ
//
void io_reg_file::rampz_write()
{
  sc_lv<6> ladr = adr.read();

  if (!nrst)
    {
      rampz = "00000000";
    }
  else
    {
      if ((ladr == RAMPZ_Address) && (iowe == 1))
	{
	  rampz = dbusout;
	}
    }
};

//
// Concurrent statements!
//
// sensitive << sreg << spl << sph << rampz;
void io_reg_file::outputregs()
{
  sc_uint<8> x = sph.read(), y = spl.read();
  sc_uint<16> z = (x,y);

  /*
  cout << "io_reg_file::outputregs()" << endl;
  cout << "stack pointer = " << sph.read() << spl.read() << " (" << (unsigned int)z << ")" << endl;
  cout << "sph_out = " << sph.read() << endl;
  cout << "spl_out = " << spl.read() << endl;
  */

  spl_out   = spl;
  sph_out   = sph;
  rampz_out = rampz;
}

//
// Set internal signals
//
// sensitive << sph << spl << sp_intm << sp_intp << sp_ndown_up;
void io_reg_file::misc()
{
  sc_lv<8> v_sph = sph;
  sc_lv<8> v_spl = spl;

  sc_lv<16> v_sp;
  sc_uint<16> i_sp;

  sc_lv<16> v_sp_res;

  v_sp = (v_sph,v_spl);
  i_sp = v_sp;
  sp_intp = i_sp + 1;
  sp_intm = i_sp - 1;

  if (sp_ndown_up == 0)
    v_sp_res = sp_intm;
  else
    v_sp_res = sp_intp;

  sp_res = v_sp_res;
}

