//
// $Id: irq_io_module.h,v 1.5 2003/11/04 10:32:11 nj Exp $
//
#ifndef _IRQ_IO_MODULE_H
#define _IRQ_IO_MODULE_H
#include <systemc.h>
#include "config.h"

#include "sleep_generator.h"
#include "halt_cpu.h"
#include "nj_mux.h"
#include "timer_counter.h"
#include "service_module.h"
#include "irq_onehot.h"
#include "array2signal23.h"

SC_MODULE(irq_io_module)
{
  sc_in<bool> clk;
  sc_in<bool> reset;

  // Tell cpu to wait
  sc_out<sc_logic> cpuwait;

  // Connections to AVR core, clock signal
  sc_out<bool> cpu_clk;

  // Connections to AVR core, interrupts
  sc_in<sc_logic> irqack;
  sc_in<sc_lv<5> > irqackad;
  sc_out<sc_lv<23> > irqlines;

  // Connections to AVR core, IO registers
  sc_in<sc_logic> ramre;
  sc_in<sc_uint<6> > adr;
  sc_in<sc_logic> iore;
  sc_in<sc_logic> iowe;
  sc_out<sc_lv<8> > dbusin;
  sc_in<sc_lv<8> > dbusout;

  // Connections to AVR core, sleep_generator
  sc_in<sc_logic> sleepi;
  // sc_in<sc_logic> irqok;
  sc_in<sc_logic> globint;

  // Connections to AVR core, dataram
  sc_out<sc_lv<8> > dbusout_ram;

  // Connections to AVR core, halt_cpu
  sc_out<sc_logic> pcwe;
  sc_in<sc_uint<16> > pc_in;
  sc_out<sc_uint<16> > pc_out;
  sc_out<sc_uint<5> > regadr;
  sc_out<sc_logic> regre;
  sc_out<sc_logic> regwe;
  sc_in<sc_lv<8> > reg_in;
  sc_out<sc_lv<8> > reg_out;


  // Signal from timer_counter to sleep_generator+output
  sc_signal<sc_lv<23> > s_irqlines;

  // Signal from sleep_generator to service_module
  sc_signal<sc_logic> sleep_en;

  // Signal from sleep_generator to halt_cpu
  sc_signal<bool> halt;

  // Signals from nj_mux to misc...
  sc_signal<sc_lv<8> > dbusout_tc;
  sc_signal<sc_lv<8> > dbusout_sm;

  // Signals for misc. interrupt routing
  sc_signal<sc_logic> x_irqlines[23];
  sc_signal<sc_logic> irqackvector[23];

  // Input to timer_counter
  sc_signal<sc_logic> out_en1;
  sc_signal<sc_logic> out_en2;
  sc_signal<sc_logic> ext1;
  sc_signal<sc_logic> ext2;

  // All signals in irqlines
  sc_signal<sc_logic> tosc1;
  sc_signal<sc_logic> oc0_pwm0;
  sc_signal<sc_logic> oc1a_pwm1a;
  sc_signal<sc_logic> oc1b_pwm1b;
  sc_signal<sc_logic> oc2_pwm2;

  sc_signal<sc_logic> tc1ovfirq;
  sc_signal<sc_logic> tc1cmpairq;
  sc_signal<sc_logic> tc1cmpbirq;
  sc_signal<sc_logic> tc1icirq;

  // Unconnected signals!!
  sc_signal<sc_logic> esram_en;
  sc_signal<sc_logic> esram_ws;
  sc_signal<sc_lv<8> > extint_irq;
  sc_signal<sc_lv<4> > extint_irq_ack;
  sc_signal<sc_lv<8> > ext_int_in;

  sleep_generator *sg;
  halt_cpu *hc;
  service_module *sm;
  irq_onehot *ioh;
  array2signal23 *as23;
  nj_mux *input_mux;
  timer_counter *tc;

  void execute();

  SC_CTOR(irq_io_module)
  {
    // Clock generator for cpu 
#ifdef WRITE_INIT
    cout << "INIT sleep_generator (sg)" << endl;
#endif
    sg = new sleep_generator("sleep_generator");
    sg->clk(clk);
    sg->reset(reset);
    sg->globint(globint);      // from avr_core
    // sg->irqok(irqok);       // from avr_core
    sg->sleep_en(sleep_en);    // from service_module
    sg->sleepi(sleepi);        // from avr_core
    sg->irqlines(s_irqlines);    // from timer_counter
    sg->halt(halt);            // to halt_cpu module
    sg->cpu_clk(cpu_clk);      // clock signal to avr_core

    // Halts the cpu completely, no clock, no power
#ifdef WRITE_INIT
    cout << "INIT halt_cpu (hc)" << endl;
#endif
    hc = new halt_cpu("halt_cpu");
    hc->clk(clk);
    hc->reset(reset);
    hc->cpuwait(cpuwait);
    hc->halt(halt);            // from sleep_generator
    // All below: Directly to avr_core
    hc->pcwe(pcwe);
    hc->pc_in(pc_in);
    hc->pc_out(pc_out);
    hc->regadr(regadr);
    hc->regre(regre);
    hc->regwe(regwe);
    hc->reg_in(reg_in);
    hc->reg_out(reg_out);

    // Service module
#ifdef WRITE_INIT
    cout << "INIT service_module (sm)" << endl;
#endif
    sm = new service_module("service_module");
    sm->cp2(clk);
    sm->ireset(reset);
    sm->adr(adr);
    sm->dbus_in(dbusin); // NB! dbusin is input to avr_core
    sm->dbus_out(dbusout_sm);
    sm->iore(iore);
    sm->iowe(iowe);
    sm->out_en(out_en1);
    sm->sleep_en(sleep_en);
    sm->esram_en(esram_en);              // not connected
    sm->esram_ws(esram_ws);              // not connected
    sm->extint_irq(extint_irq);          // not connected
    sm->extint_irq_ack(extint_irq_ack);  // not connected
    sm->ext_int_in(ext_int_in);          // not connected

#ifdef WRITE_INIT
    cout << "INIT irq_onehot (ioh)" << endl;
#endif
    ioh = new irq_onehot("irq_onehot");
    ioh->irqack(irqack);
    ioh->irqackad(irqackad);
    ioh->irq_onehot_ack0(irqackvector[0]);
    ioh->irq_onehot_ack1(irqackvector[1]);
    ioh->irq_onehot_ack2(irqackvector[2]);
    ioh->irq_onehot_ack3(irqackvector[3]);
    ioh->irq_onehot_ack4(irqackvector[4]);
    ioh->irq_onehot_ack5(irqackvector[5]);
    ioh->irq_onehot_ack6(irqackvector[6]);
    ioh->irq_onehot_ack7(irqackvector[7]);
    ioh->irq_onehot_ack8(irqackvector[8]);
    ioh->irq_onehot_ack9(irqackvector[9]);
    ioh->irq_onehot_ack10(irqackvector[10]);
    ioh->irq_onehot_ack11(irqackvector[11]);
    ioh->irq_onehot_ack12(irqackvector[12]);
    ioh->irq_onehot_ack13(irqackvector[13]);
    ioh->irq_onehot_ack14(irqackvector[14]);
    ioh->irq_onehot_ack15(irqackvector[15]);
    ioh->irq_onehot_ack16(irqackvector[16]);
    ioh->irq_onehot_ack17(irqackvector[17]);
    ioh->irq_onehot_ack18(irqackvector[18]);
    ioh->irq_onehot_ack19(irqackvector[19]);
    ioh->irq_onehot_ack20(irqackvector[20]);
    ioh->irq_onehot_ack21(irqackvector[21]);
    ioh->irq_onehot_ack22(irqackvector[22]);

#ifdef WRITE_INIT
    cout << "INIT array2signal (as23)" << endl;
#endif
    as23 = new array2signal23("array2signal23");
    as23->in0(x_irqlines[0]);
    as23->in1(x_irqlines[1]);
    as23->in2(x_irqlines[2]);
    as23->in3(x_irqlines[3]);
    as23->in4(x_irqlines[4]);
    as23->in5(x_irqlines[5]);
    as23->in6(x_irqlines[6]);
    as23->in7(x_irqlines[7]);
    as23->in8(x_irqlines[8]);
    as23->in9(x_irqlines[9]);
    as23->in10(x_irqlines[10]);
    as23->in11(x_irqlines[11]);
    as23->in12(x_irqlines[12]);
    as23->in13(x_irqlines[13]);
    as23->in14(x_irqlines[14]);
    as23->in15(x_irqlines[15]);
    as23->in16(x_irqlines[16]);
    as23->in17(x_irqlines[17]);
    as23->in18(x_irqlines[18]);
    as23->in19(x_irqlines[19]);
    as23->in20(x_irqlines[20]);
    as23->in21(x_irqlines[21]);
    as23->in22(x_irqlines[22]);
    as23->out(s_irqlines);

#ifdef WRITE_INIT
    cout << "INIT njmux (input_mux)" << endl;
#endif
    input_mux = new nj_mux("mux_io_registers");
    input_mux->clk(clk);
    input_mux->adr(adr);
    input_mux->iore(iore);
    input_mux->iowe(iowe);
    input_mux->ramre(ramre);
    input_mux->dbusout(dbusin);         // dbusin  on avr_core
    input_mux->dbusin_ram(dbusout_ram); // dbusout on dataram
    input_mux->dbusin_avr(dbusout);     // dbusout on avr_core
    input_mux->dbusin_tc(dbusout_tc);   // dbusout on timer_counter
    input_mux->dbusin_sm(dbusout_sm);   // dbusout on service_module

#ifdef WRITE_INIT
    cout << "INIT timer_counter (tc)" << endl;
#endif
    tc = new timer_counter("timer_counter");
    tc->clk(clk);
    tc->reset(reset);
    tc->adr(adr);
    tc->dbus_in(dbusout); // NB! dbusout from avr_core
    tc->dbus_out(dbusout_tc);
    tc->iore(iore);
    tc->iowe(iowe);
    tc->out_en(out_en2);
    
    // IRQ lines
    // These 7 lines are just connected at random
    tc->ext1(ext1);
    tc->ext2(ext2);
    tc->tosc1(tosc1);
    tc->oc0_pwm0(oc0_pwm0);
    tc->oc1a_pwm1a(oc1a_pwm1a);
    tc->oc1b_pwm1b(oc1b_pwm1b);
    tc->oc2_pwm2(oc2_pwm2);

    // These lines should be okay
    tc->tc0ovfirq(x_irqlines[16]);
    tc->tc0ovfirq_ack(irqackvector[16]);
    tc->tc0cmpirq(x_irqlines[15]);
    tc->tc0cmpirq_ack(irqackvector[15]);
    /*
      tc->tc0ovfirq(x_irqlines[15]);
      tc->tc0ovfirq_ack(irqackvector[15]);
      tc->tc0cmpirq(x_irqlines[14]);
      tc->tc0cmpirq_ack(irqackvector[14]);
    */
    
    tc->tc2ovfirq(x_irqlines[9]);
    tc->tc2ovfirq_ack(irqackvector[9]);
    tc->tc2cmpirq(x_irqlines[8]);
    tc->tc2cmpirq_ack(irqackvector[8]);

    // The rest of the lines are just connected at random
    tc->tc1ovfirq(x_irqlines[19]);
    tc->tc1ovfirq_ack(irqackvector[10]);
    tc->tc1cmpairq(x_irqlines[20]);
    tc->tc1cmpairq_ack(irqackvector[20]);
    tc->tc1cmpbirq(x_irqlines[21]);
    tc->tc1cmpbirq_ack(irqackvector[21]);
    tc->tc1icirq(x_irqlines[22]);
    tc->tc1icirq_ack(irqackvector[22]);

    x_irqlines[0] = SC_LOGIC_0;
    x_irqlines[1] = SC_LOGIC_0;
    x_irqlines[2] = SC_LOGIC_0;
    x_irqlines[3] = SC_LOGIC_0;
    x_irqlines[4] = SC_LOGIC_0;
    x_irqlines[5] = SC_LOGIC_0;
    x_irqlines[6] = SC_LOGIC_0;
    x_irqlines[7] = SC_LOGIC_0;
    x_irqlines[10] = SC_LOGIC_0;
    x_irqlines[11] = SC_LOGIC_0;
    x_irqlines[12] = SC_LOGIC_0;
    x_irqlines[13] = SC_LOGIC_0;
    x_irqlines[14] = SC_LOGIC_0;
    x_irqlines[17] = SC_LOGIC_0;
    x_irqlines[18] = SC_LOGIC_0;
    
    // to timer_counter inputs
    ext1   = SC_LOGIC_0;
    ext2   = SC_LOGIC_0;
    tosc1  = SC_LOGIC_0;
    
    // No external interrupts
    ext_int_in = "00000000";

    SC_METHOD(execute);
    sensitive << s_irqlines; // << override_inst;
  }
};

#endif
