//
// $Id: main.cc,v 1.42 2003/11/05 16:16:23 nj Exp $
//
#include <systemc.h>
#include <stdlib.h>
#include "config.h"
#include "constants.h"
#include "oneshot.h"
#include "avr_core.h"
#include "irq_io_module.h"
#include "instmemory.h"
#include "dataram.h"

int sc_main(int argc, char *argv[])
{
  sc_clock clk("clock", 20, SC_NS);
  sc_signal<bool> reset;

  if (argc == 1)
    {
      cout << "Usage:" << endl;
      cout << argv[0] << " <bin-file> [runlength (ns)]" << endl;
      return 0;
    }

  /////////////////////////////////////////////////////////////////
  // Signals to/from avr_core
  /////////////////////////////////////////////////////////////////
  sc_signal<bool> cpu_clk;
  sc_signal<sc_logic> cpuwait;
  // sc_signal<sc_logic> xx_cpuwait;
  sc_signal<sc_uint<16> > pc;
  sc_signal<sc_lv<16> > inst;
  sc_signal<sc_uint<6> > adr;
  sc_signal<sc_logic> iore;
  sc_signal<sc_logic> iowe;
  sc_signal<sc_uint<16> > ramadr;
  sc_signal<sc_logic> ramre;
  sc_signal<sc_logic> ramwe;
  sc_signal<sc_lv<8> > dbusin;
  sc_signal<sc_lv<8> > dbusout;

  sc_signal<sc_logic> sleepi;
  sc_signal<sc_logic> irqok;
  sc_signal<sc_logic> globint;

  sc_signal<sc_lv<23> > irqlines;
  sc_signal<sc_logic> irqack;
  sc_signal<sc_lv<5> > irqackad;

  // halt_signals
  sc_signal<sc_logic> pcwe;
  sc_signal<sc_uint<16> > pc_in;
  sc_signal<sc_uint<16> > pc_out;

  sc_signal<sc_uint<5> > regadr;
  sc_signal<sc_logic> regre;
  sc_signal<sc_logic> regwe;
  sc_signal<sc_lv<8> > reg_in;
  sc_signal<sc_lv<8> > reg_out;

  // Input to timer_counter
  sc_signal<sc_logic> out_en;

  sc_signal<sc_logic> ext1;
  sc_signal<sc_logic> ext2;

  // All signals in irqlines
  // sc_signal<sc_logic> tosc1;
  sc_signal<sc_logic> oc0_pwm0;
  sc_signal<sc_logic> oc1a_pwm1a;
  sc_signal<sc_logic> oc1b_pwm1b;
  sc_signal<sc_logic> oc2_pwm2;

  sc_signal<sc_logic> tc1ovfirq;
  sc_signal<sc_logic> tc1cmpairq;
  sc_signal<sc_logic> tc1cmpbirq;
  sc_signal<sc_logic> tc1icirq;

  sc_signal<sc_lv<8> > dbusout_ram;


  /////////////////////////////////////////////////////////////////
  // Init components
  /////////////////////////////////////////////////////////////////
#ifdef WRITE_INIT
  cout << "INIT oneshot (oneshot1)" << endl;
#endif
  oneshot *oneshot1;
  oneshot1 = new oneshot("reset-blok");
  oneshot1->clk(clk.signal());
  oneshot1->reset(reset);

#ifdef WRITE_INIT
  cout << "INIT avr_core (cpu)" << endl;
#endif
  avr_core *cpu;
  cpu = new avr_core("avr_core");
  cpu->cp2(cpu_clk);
  cpu->ireset(reset);
  cpu->cpuwait(cpuwait);
  cpu->pc(pc);
  cpu->inst(inst);
  cpu->adr(adr);
  cpu->iore(iore);
  cpu->iowe(iowe);
  cpu->ramadr(ramadr);
  cpu->ramre(ramre);
  cpu->ramwe(ramwe);
  cpu->dbusin(dbusin);
  cpu->dbusout(dbusout);
  cpu->sleepi(sleepi);
  cpu->irqok(irqok);
  cpu->globint(globint); // unused...
  cpu->irqlines(irqlines);
  cpu->irqack(irqack);
  cpu->irqackad(irqackad);
  cpu->pcwe(pcwe);
  cpu->pc_in(pc_in);
  cpu->pc_out(pc_out);
  cpu->regadr(regadr);
  cpu->regre(regre);
  cpu->regwe(regwe);
  cpu->reg_in(reg_in);
  cpu->reg_out(reg_out);

#ifdef WRITE_INIT
  cout << "INIT irq_io_modulec (irq_io)" << endl;
#endif
  irq_io_module *irq_io;
  irq_io = new irq_io_module("irq_io_module");
  irq_io->clk(clk.signal());
  irq_io->reset(reset);
  irq_io->cpuwait(cpuwait);
  irq_io->cpu_clk(cpu_clk);
  irq_io->irqack(irqack);
  irq_io->irqackad(irqackad);
  irq_io->irqlines(irqlines);
  irq_io->ramre(ramre);
  irq_io->adr(adr);
  irq_io->iore(iore);
  irq_io->iowe(iowe);
  irq_io->dbusin(dbusin);
  irq_io->dbusout(dbusout);
  irq_io->sleepi(sleepi);
  // irq_io->irqok(irqok);
  irq_io->globint(globint);
  irq_io->dbusout_ram(dbusout_ram);

  irq_io->pcwe(pcwe);
  irq_io->pc_in(pc_in);
  irq_io->pc_out(pc_out);

  irq_io->regadr(regadr);
  irq_io->regre(regre);
  irq_io->regwe(regwe);
  irq_io->reg_in(reg_in);
  irq_io->reg_out(reg_out);

#ifdef WRITE_INIT
  cout << "INIT instmemory (imem)" << endl;
#endif
  instmemory *imem;
  imem = new instmemory("imemory", argv[1]);
  imem->clk(clk.signal());
  imem->reset(reset);
  imem->pc(pc);
  imem->inst(inst);

#ifdef WRITE_INIT
  cout << "INIT dataram (dmem)" << endl;
#endif
  dataram *dmem;
  dmem = new dataram("dmemory", argv[1]);
  dmem->clk(clk.signal());
  dmem->ramadr(ramadr);
  dmem->ramre(ramre);
  dmem->ramwe(ramwe);
  dmem->dbusin(dbusout); // NB! dbusout from avr_core
  dmem->dbusout(dbusout_ram);


  /////////////////////////////////////////////////////////////////
  // Set constant values
  /////////////////////////////////////////////////////////////////
  // cpuwait  = SC_LOGIC_0;

  /*
  x_irqlines[0] = SC_LOGIC_0;
  x_irqlines[1] = SC_LOGIC_0;
  x_irqlines[2] = SC_LOGIC_0;
  x_irqlines[3] = SC_LOGIC_0;
  x_irqlines[4] = SC_LOGIC_0;
  x_irqlines[5] = SC_LOGIC_0;
  x_irqlines[6] = SC_LOGIC_0;
  x_irqlines[7] = SC_LOGIC_0;
  x_irqlines[10] = SC_LOGIC_0;
  x_irqlines[11] = SC_LOGIC_0;
  x_irqlines[12] = SC_LOGIC_0;
  x_irqlines[13] = SC_LOGIC_0;
  x_irqlines[14] = SC_LOGIC_0;
  x_irqlines[17] = SC_LOGIC_0;
  x_irqlines[18] = SC_LOGIC_0;

  // to timer_counter inputs
  ext1   = SC_LOGIC_0;
  ext2   = SC_LOGIC_0;
  tosc1  = SC_LOGIC_0;

  // No external interrupts
  ext_int_in = "00000000";
  */

  /////////////////////////////////////////////////////////////////
  // Trace file - VCD format...
  /////////////////////////////////////////////////////////////////

  sc_trace_file *trace_file;
  trace_file = sc_create_vcd_trace_file("main.trace");

#ifdef SIGNAL_AVR_CORE
  // AVR_CORE
  sc_trace(trace_file, cpu_clk, "cp2");
  sc_trace(trace_file, reset, "ireset");

  // Connections to avr_core
  sc_trace(trace_file, cpuwait, "cpuwait");
  sc_trace(trace_file, pc, "pc");
  sc_trace(trace_file, inst, "inst");
  sc_trace(trace_file, adr, "adr");
  sc_trace(trace_file, iore, "iore");
  sc_trace(trace_file, iowe, "iowe");
  sc_trace(trace_file, ramadr, "ramadr");
  sc_trace(trace_file, ramre, "ramre");
  sc_trace(trace_file, ramwe, "ramwe");
  sc_trace(trace_file, dbusin, "dbusin");
  sc_trace(trace_file, dbusout, "dbusout");
  sc_trace(trace_file, sleepi, "sleepi");
  sc_trace(trace_file, irqok, "irqok");
  sc_trace(trace_file, globint, "globint");
  sc_trace(trace_file, irqlines, "irqlines");
  sc_trace(trace_file, irqack, "irqack");
  sc_trace(trace_file, irqackad, "irqackad");
  sc_trace(trace_file, pcwe, "pcwe");
  sc_trace(trace_file, pc_in, "pc_in");
  sc_trace(trace_file, pc_out, "pc_out");
  sc_trace(trace_file, regadr, "regadr");
  sc_trace(trace_file, regre, "regre");
  sc_trace(trace_file, regwe, "regwe");
  sc_trace(trace_file, reg_in, "reg_in");
  sc_trace(trace_file, reg_out, "reg_out");
#endif

#ifdef SIGNAL_AVR_CORE_TO_PM_FETCH_DEC
  // All connections to/from AVR_CORE

  sc_trace(trace_file, clk.signal(), "clk");
  sc_trace(trace_file, reset, "reset");

  sc_trace(trace_file, cpu->sg_dbusin, "cpu.sg_dbusin");
  sc_trace(trace_file, cpu->sg_dbusout, "cpu.sg_dbusout");
  sc_trace(trace_file, cpu->sg_adr, "cpu.sg_adr");
  sc_trace(trace_file, cpu->sg_iowe, "cpu.sg_iowe");
  sc_trace(trace_file, cpu->sg_iore, "cpu.sg_iore");
  sc_trace(trace_file, cpu->sg_idc_adc, "cpu.sg_idc_adc");
  sc_trace(trace_file, cpu->sg_idc_add, "cpu.sg_idc_add");
  sc_trace(trace_file, cpu->sg_idc_adiw, "cpu.sg_idc_adiw");
  sc_trace(trace_file, cpu->sg_idc_sub, "cpu.sg_idc_sub");
  sc_trace(trace_file, cpu->sg_idc_subi, "cpu.sg_idc_subi");
  sc_trace(trace_file, cpu->sg_idc_sbc, "cpu.sg_idc_sbc");
  sc_trace(trace_file, cpu->sg_idc_sbci, "cpu.sg_idc_sbci");
  sc_trace(trace_file, cpu->sg_idc_sbiw, "cpu.sg_idc_sbiw");
  sc_trace(trace_file, cpu->sg_adiw_st, "cpu.sg_adiw_st");
  sc_trace(trace_file, cpu->sg_sbiw_st, "cpu.sg_sbiw_st");
  sc_trace(trace_file, cpu->sg_idc_and, "cpu.sg_idc_and");
  sc_trace(trace_file, cpu->sg_idc_andi, "cpu.sg_idc_andi");
  sc_trace(trace_file, cpu->sg_idc_or, "cpu.sg_idc_or");
  sc_trace(trace_file, cpu->sg_idc_ori, "cpu.sg_idc_ori");
  sc_trace(trace_file, cpu->sg_idc_eor, "cpu.sg_idc_eor");
  sc_trace(trace_file, cpu->sg_idc_com, "cpu.sg_idc_com");
  sc_trace(trace_file, cpu->sg_idc_neg, "cpu.sg_idc_neg");
  sc_trace(trace_file, cpu->sg_idc_inc, "cpu.sg_idc_inc");
  sc_trace(trace_file, cpu->sg_idc_dec, "cpu.sg_idc_dec");
  sc_trace(trace_file, cpu->sg_idc_cp, "cpu.sg_idc_cp");
  sc_trace(trace_file, cpu->sg_idc_cpc, "cpu.sg_idc_cpc");
  sc_trace(trace_file, cpu->sg_idc_cpi, "cpu.sg_idc_cpi");
  sc_trace(trace_file, cpu->sg_idc_cpse, "cpu.sg_idc_cpse");
  sc_trace(trace_file, cpu->sg_idc_lsr, "cpu.sg_idc_lsr");
  sc_trace(trace_file, cpu->sg_idc_ror, "cpu.sg_idc_ror");
  sc_trace(trace_file, cpu->sg_idc_asr, "cpu.sg_idc_asr");
  sc_trace(trace_file, cpu->sg_idc_swap, "cpu.sg_idc_swap");
  sc_trace(trace_file, cpu->sg_idc_sbi, "cpu.sg_idc_sbi");
  // sc_trace(trace_file, cpu->sg_idc_st, "cpu.sg_idc_st");
  sc_trace(trace_file, cpu->sg_idc_bld, "cpu.sg_idc_bld");
  sc_trace(trace_file, cpu->sg_idc_bst, "cpu.sg_idc_bst");
  sc_trace(trace_file, cpu->sg_idc_bset, "cpu.sg_idc_bset");
  sc_trace(trace_file, cpu->sg_idc_bclr, "cpu.sg_idc_bclr");
  sc_trace(trace_file, cpu->sg_idc_sbic, "cpu.sg_idc_sbic");
  sc_trace(trace_file, cpu->sg_idc_sbis, "cpu.sg_idc_sbis");
  sc_trace(trace_file, cpu->sg_idc_sbrs, "cpu.sg_idc_sbrs");
  sc_trace(trace_file, cpu->sg_idc_sbrc, "cpu.sg_idc_sbrc");
  sc_trace(trace_file, cpu->sg_idc_brbs, "cpu.sg_idc_brbs");
  sc_trace(trace_file, cpu->sg_idc_brbc, "cpu.sg_idc_brbc");
  sc_trace(trace_file, cpu->sg_idc_reti, "cpu.sg_idc_reti");

  sc_trace(trace_file, cpu->sg_alu_data_r_in, "cpu.sg_alu_data_r_in");
  sc_trace(trace_file, cpu->sg_alu_data_d_in, "cpu.sg_alu_data_d_in");
  sc_trace(trace_file, cpu->sg_alu_data_out, "cpu.sg_alu_data_out");
  sc_trace(trace_file, cpu->sg_reg_rd_in, "cpu.sg_reg_rd_in");
  sc_trace(trace_file, cpu->sg_reg_rd_out, "cpu.sg_reg_rd_out");
  sc_trace(trace_file, cpu->sg_reg_rr_out, "cpu.sg_reg_rr_out");
  sc_trace(trace_file, cpu->sg_reg_rd_adr, "cpu.sg_reg_rd_adr");
  sc_trace(trace_file, cpu->sg_reg_rr_adr, "cpu.sg_reg_rr_adr");
  sc_trace(trace_file, cpu->sg_reg_h_out, "cpu.sg_reg_h_out");
  sc_trace(trace_file, cpu->sg_reg_z_out, "cpu.sg_reg_z_out");
  sc_trace(trace_file, cpu->sg_reg_h_adr, "cpu.sg_reg_h_adr");
  sc_trace(trace_file, cpu->sg_reg_rd_wr, "cpu.sg_reg_rd_wr");
  sc_trace(trace_file, cpu->sg_post_inc, "cpu.sg_post_inc");
  sc_trace(trace_file, cpu->sg_pre_dec, "cpu.sg_pre_dec");
  sc_trace(trace_file, cpu->sg_reg_h_wr, "cpu.sg_reg_h_wr");
  sc_trace(trace_file, cpu->sg_sreg_fl_in, "cpu.sg_sreg_fl_in");
  sc_trace(trace_file, cpu->sg_sreg_out, "cpu.sg_sreg_out");
  sc_trace(trace_file, cpu->sg_sreg_fl_wr_en, "cpu.sg_sreg_fl_wr_en");
  sc_trace(trace_file, cpu->sg_spl_out, "cpu.sg_spl_out");
  sc_trace(trace_file, cpu->sg_sph_out, "cpu.sg_sph_out");
  sc_trace(trace_file, cpu->sg_rampz_out, "cpu.sg_rampz_out");
  sc_trace(trace_file, cpu->sg_sp_ndown_up, "cpu.sg_sp_ndown_up");
  sc_trace(trace_file, cpu->sg_sp_en, "cpu.sg_sp_en");

  sc_trace(trace_file, cpu->sg_bit_test_op_out, "cpu.sg_bit_test_op_out");
  sc_trace(trace_file, cpu->sg_alu_c_flag_out, "cpu.sg_alu_c_flag_out");
  sc_trace(trace_file, cpu->sg_alu_z_flag_out, "cpu.sg_alu_z_flag_out");
  sc_trace(trace_file, cpu->sg_alu_n_flag_out, "cpu.sg_alu_n_flag_out");
  sc_trace(trace_file, cpu->sg_alu_v_flag_out, "cpu.sg_alu_v_flag_out");
  sc_trace(trace_file, cpu->sg_alu_s_flag_out, "cpu.sg_alu_s_flag_out");
  sc_trace(trace_file, cpu->sg_alu_h_flag_out, "cpu.sg_alu_h_flag_out");

  sc_trace(trace_file, cpu->sg_sleepi, "cpu.sg_sleepi");
#endif


#ifdef SIGNAL_PM_FETCH_DEC
  sc_trace(trace_file, cpu->maincore->clk, "clk");
  sc_trace(trace_file, cpu->maincore->nrst, "nrst");

  sc_trace(trace_file, cpu->maincore->pc, "pc");
  sc_trace(trace_file, cpu->maincore->cpuwait, "cpuwait");
  sc_trace(trace_file, cpu->maincore->inst, "inst");
  sc_trace(trace_file, cpu->maincore->adr, "adr");
  sc_trace(trace_file, cpu->maincore->iore, "iore");
  sc_trace(trace_file, cpu->maincore->iowe, "iowe");
  sc_trace(trace_file, cpu->maincore->ramadr, "ramadr");
  sc_trace(trace_file, cpu->maincore->ramre, "ramre");
  sc_trace(trace_file, cpu->maincore->ramwe, "ramwe");
  sc_trace(trace_file, cpu->maincore->dbusin, "dbusin");
  sc_trace(trace_file, cpu->maincore->dbusout, "dbusout");
  sc_trace(trace_file, cpu->maincore->irqlines, "irqlines");
  sc_trace(trace_file, cpu->maincore->irqack, "irqack");
  sc_trace(trace_file, cpu->maincore->irqackad, "irqackad");
  sc_trace(trace_file, cpu->maincore->alu_data_r_in, "alu_data_r_in");
  sc_trace(trace_file, cpu->maincore->alu_data_d_in, "alu_data_d_in");

  sc_trace(trace_file, cpu->maincore->idc_add_out, "idc_add_out");
  sc_trace(trace_file, cpu->maincore->idc_adc_out, "idc_adc_out");
  sc_trace(trace_file, cpu->maincore->idc_adiw_out, "idc_adiw_out");
  sc_trace(trace_file, cpu->maincore->idc_sub_out, "idc_sub_out");
  sc_trace(trace_file, cpu->maincore->idc_subi_out, "idc_subi_out");
  sc_trace(trace_file, cpu->maincore->idc_sbc_out, "idc_sbc_out");
  sc_trace(trace_file, cpu->maincore->idc_sbci_out, "idc_sbci_out");
  sc_trace(trace_file, cpu->maincore->idc_sbiw_out, "idc_sbiw_out");
  sc_trace(trace_file, cpu->maincore->adiw_st_out, "adiw_st_out");
  sc_trace(trace_file, cpu->maincore->sbiw_st_out, "sbiw_st_out");
  sc_trace(trace_file, cpu->maincore->idc_and_out, "idc_and_out");
  sc_trace(trace_file, cpu->maincore->idc_andi_out, "idc_andi_out");
  sc_trace(trace_file, cpu->maincore->idc_or_out, "idc_or_out");
  sc_trace(trace_file, cpu->maincore->idc_ori_out, "idc_ori_out");
  sc_trace(trace_file, cpu->maincore->idc_eor_out, "idc_eor_out");
  sc_trace(trace_file, cpu->maincore->idc_com_out, "idc_com_out");
  sc_trace(trace_file, cpu->maincore->idc_neg_out, "idc_neg_out");
  sc_trace(trace_file, cpu->maincore->idc_inc_out, "idc_inc_out");
  sc_trace(trace_file, cpu->maincore->idc_dec_out, "idc_dec_out");
  sc_trace(trace_file, cpu->maincore->idc_cp_out, "idc_cp_out");
  sc_trace(trace_file, cpu->maincore->idc_cpc_out, "idc_cpc_out");
  sc_trace(trace_file, cpu->maincore->idc_cpi_out, "idc_cpi_out");
  sc_trace(trace_file, cpu->maincore->idc_cpse_out, "idc_cpse_out");
  sc_trace(trace_file, cpu->maincore->idc_lsr_out, "idc_lsr_out");
  sc_trace(trace_file, cpu->maincore->idc_ror_out, "idc_ror_out");
  sc_trace(trace_file, cpu->maincore->idc_asr_out, "idc_asr_out");
  sc_trace(trace_file, cpu->maincore->idc_swap_out, "idc_swap_out");

  sc_trace(trace_file, cpu->maincore->alu_data_out, "alu_data_out");
  sc_trace(trace_file, cpu->maincore->alu_c_flag_out, "alu_c_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_z_flag_out, "alu_z_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_n_flag_out, "alu_n_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_v_flag_out, "alu_v_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_s_flag_out, "alu_s_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_h_flag_out, "alu_h_flag_out");

  sc_trace(trace_file, cpu->maincore->reg_rd_in, "reg_rd_in");
  sc_trace(trace_file, cpu->maincore->reg_rd_out, "reg_rd_out");
  sc_trace(trace_file, cpu->maincore->reg_rd_adr, "reg_rd_adr");
  sc_trace(trace_file, cpu->maincore->reg_rr_out, "reg_rr_out");
  sc_trace(trace_file, cpu->maincore->reg_rr_adr, "reg_rr_adr");
  sc_trace(trace_file, cpu->maincore->reg_rd_wr, "reg_rd_wr");
  sc_trace(trace_file, cpu->maincore->post_inc, "post_inc");
  sc_trace(trace_file, cpu->maincore->pre_dec, "pre_dec");
  sc_trace(trace_file, cpu->maincore->reg_h_wr, "reg_h_wr");
  sc_trace(trace_file, cpu->maincore->reg_h_out, "reg_h_out");
  sc_trace(trace_file, cpu->maincore->reg_h_adr, "reg_h_adr");
  sc_trace(trace_file, cpu->maincore->reg_z_out, "reg_z_out");

  sc_trace(trace_file, cpu->maincore->sreg_fl_in, "sreg_fl_in");
  sc_trace(trace_file, cpu->maincore->sreg_out, "sreg_out");
  sc_trace(trace_file, cpu->maincore->sreg_fl_wr_en, "sreg_fl_wr_en");
  sc_trace(trace_file, cpu->maincore->spl_out, "spl_out");
  sc_trace(trace_file, cpu->maincore->sph_out, "sph_out");
  sc_trace(trace_file, cpu->maincore->sp_ndown_up, "sp_ndown_up");
  sc_trace(trace_file, cpu->maincore->sp_en, "sp_en");
  sc_trace(trace_file, cpu->maincore->rampz_out, "rampz_out");

  sc_trace(trace_file, cpu->maincore->bit_num_r_io, "bit_num_r_io");
  sc_trace(trace_file, cpu->maincore->bitpr_io_out, "bitpr_io_out");
  sc_trace(trace_file, cpu->maincore->branch, "branch");
  sc_trace(trace_file, cpu->maincore->bit_pr_sreg_out, "bit_pr_sreg_out");
  sc_trace(trace_file, cpu->maincore->sreg_bit_num, "sreg_bit_num");
  sc_trace(trace_file, cpu->maincore->bld_op_out, "bld_op_out");
  sc_trace(trace_file, cpu->maincore->bit_test_op_out, "bit_test_op_out");
  sc_trace(trace_file, cpu->maincore->idc_sbi_out, "idc_sbi_out");
  sc_trace(trace_file, cpu->maincore->sbi_st_out, "sbi_st_out");
  sc_trace(trace_file, cpu->maincore->idc_cbi_out, "idc_cbi_out");
  sc_trace(trace_file, cpu->maincore->cbi_st_out, "cbi_st_out");

  sc_trace(trace_file, cpu->maincore->idc_bld_out, "idc_bld_out");
  sc_trace(trace_file, cpu->maincore->idc_bst_out, "idc_bst_out");
  sc_trace(trace_file, cpu->maincore->idc_bset_out, "idc_bset_out");
  sc_trace(trace_file, cpu->maincore->idc_bclr_out, "idc_bclr_out");
  sc_trace(trace_file, cpu->maincore->idc_sbic_out, "idc_sbic_out");
  sc_trace(trace_file, cpu->maincore->idc_sbis_out, "idc_sbis_out");
  sc_trace(trace_file, cpu->maincore->idc_sbrs_out, "idc_sbrs_out");
  sc_trace(trace_file, cpu->maincore->idc_sbrc_out, "idc_sbrc_out");
  sc_trace(trace_file, cpu->maincore->idc_brbs_out, "idc_brbs_out");
  sc_trace(trace_file, cpu->maincore->idc_brbc_out, "idc_brbc_out");
  sc_trace(trace_file, cpu->maincore->idc_reti_out, "idc_reti_out");
  sc_trace(trace_file, cpu->maincore->sleepi, "sleepi");
  sc_trace(trace_file, cpu->maincore->pcwe, "pcwe");
  sc_trace(trace_file, cpu->maincore->pc_in, "pc_in");
  sc_trace(trace_file, cpu->maincore->pc_out, "pc_out");
#endif


#ifdef SIGNAL_BIT_PROCESSOR
  // BIT_PROCESSOR
  sc_trace(trace_file, cpu->bit_proc->clk, "clk");
  sc_trace(trace_file, cpu->bit_proc->nrst, "nrst");

  sc_trace(trace_file, cpu->bit_proc->bit_num_r_io, "bit_num_r_io");
  sc_trace(trace_file, cpu->bit_proc->dbusin, "dbusin");
  sc_trace(trace_file, cpu->bit_proc->bitpr_io_out, "bitpr_io_out");
  sc_trace(trace_file, cpu->bit_proc->sreg_out, "sreg_out");
  sc_trace(trace_file, cpu->bit_proc->branch, "branch");

  sc_trace(trace_file, cpu->bit_proc->bit_pr_sreg_out, "bit_pr_sreg_out");
  sc_trace(trace_file, cpu->bit_proc->sreg_bit_num, "sreg_bit_num");
  sc_trace(trace_file, cpu->bit_proc->bld_op_out, "bld_op_out");
  sc_trace(trace_file, cpu->bit_proc->reg_rd_out, "reg_rd_out");
  sc_trace(trace_file, cpu->bit_proc->bit_test_op_out, "bit_test_op_out");

  sc_trace(trace_file, cpu->bit_proc->idc_sbi, "idc_sbi");
  sc_trace(trace_file, cpu->bit_proc->sbi_st, "sbi_st");
  sc_trace(trace_file, cpu->bit_proc->idc_cbi, "idc_cbi");
  sc_trace(trace_file, cpu->bit_proc->cbi_st, "cbi_st");
  sc_trace(trace_file, cpu->bit_proc->idc_bld, "idc_bld");
  sc_trace(trace_file, cpu->bit_proc->idc_bst, "idc_bst");
  sc_trace(trace_file, cpu->bit_proc->idc_bset, "idc_bset");

  sc_trace(trace_file, cpu->bit_proc->idc_bclr, "idc_bclr");
  sc_trace(trace_file, cpu->bit_proc->idc_sbic, "idc_sbic");
  sc_trace(trace_file, cpu->bit_proc->idc_sbis, "idc_sbis");
  sc_trace(trace_file, cpu->bit_proc->idc_sbrs, "idc_sbrs");
  sc_trace(trace_file, cpu->bit_proc->idc_sbrc, "idc_sbrc");
  sc_trace(trace_file, cpu->bit_proc->idc_brbs, "idc_brbs");
  sc_trace(trace_file, cpu->bit_proc->idc_brbc, "idc_brbc");
  sc_trace(trace_file, cpu->bit_proc->idc_reti, "idc_reti");
#endif

#ifdef SIGNAL_REG_FILE
  // REG_FILE
  sc_trace(trace_file, cpu->gp_register_file->clk, "clk");
  sc_trace(trace_file, cpu->gp_register_file->nrst, "nrst");

  sc_trace(trace_file, cpu->gp_register_file->reg_rd_in, "reg_rd_in");
  sc_trace(trace_file, cpu->gp_register_file->reg_rd_out, "reg_rd_out");
  sc_trace(trace_file, cpu->gp_register_file->reg_rd_adr, "reg_rd_adr");
  sc_trace(trace_file, cpu->gp_register_file->reg_rr_out, "reg_rr_out");
  sc_trace(trace_file, cpu->gp_register_file->reg_rr_adr, "reg_rr_adr");
  sc_trace(trace_file, cpu->gp_register_file->reg_rd_wr, "reg_rd_wr");
  sc_trace(trace_file, cpu->gp_register_file->post_inc, "post_inc");
  sc_trace(trace_file, cpu->gp_register_file->pre_dec, "pre_dec");
  sc_trace(trace_file, cpu->gp_register_file->reg_h_wr, "reg_h_wr");
  sc_trace(trace_file, cpu->gp_register_file->reg_h_out, "reg_h_out");
  sc_trace(trace_file, cpu->gp_register_file->reg_h_adr, "reg_h_adr");
  sc_trace(trace_file, cpu->gp_register_file->reg_z_out, "reg_z_out");

  sc_trace(trace_file, cpu->gp_register_file->regadr, "regadr");
  sc_trace(trace_file, cpu->gp_register_file->regre, "regre");
  sc_trace(trace_file, cpu->gp_register_file->regwe, "regwe");
  sc_trace(trace_file, cpu->gp_register_file->reg_in, "reg_in");
  sc_trace(trace_file, cpu->gp_register_file->reg_out, "reg_out");
#endif

#ifdef SIGNAL_IO_ADR_DEC
  sc_trace(trace_file, cpu->io_dec->adr, "adr");
  sc_trace(trace_file, cpu->io_dec->iore, "iore");
  sc_trace(trace_file, cpu->io_dec->dbusin_ext, "dbusin_ext");
  sc_trace(trace_file, cpu->io_dec->dbusin_int, "dbusin_int");
  sc_trace(trace_file, cpu->io_dec->spl_out, "spl_out");
  sc_trace(trace_file, cpu->io_dec->sph_out, "sph_out");
  sc_trace(trace_file, cpu->io_dec->sreg_out, "sreg_out");
  sc_trace(trace_file, cpu->io_dec->rampz_out, "rampz_out");
#endif

#ifdef SIGNAL_IO_REG_FILE
  // IO_REG_FILE
  sc_trace(trace_file, cpu->io_registers->clk, "clk");
  sc_trace(trace_file, cpu->io_registers->nrst, "nrst");

  sc_trace(trace_file, cpu->io_registers->adr, "adr");
  sc_trace(trace_file, cpu->io_registers->iowe, "iowe");
  sc_trace(trace_file, cpu->io_registers->dbusout, "dbusout");
  sc_trace(trace_file, cpu->io_registers->sreg_fl_in, "sreg_fl_in");
  sc_trace(trace_file, cpu->io_registers->sreg_out, "sreg_out");
  sc_trace(trace_file, cpu->io_registers->sreg_fl_wr_en, "sreg_fl_wr_en");
  sc_trace(trace_file, cpu->io_registers->spl_out, "spl_out");
  sc_trace(trace_file, cpu->io_registers->sph_out, "sph_out");
  sc_trace(trace_file, cpu->io_registers->sp_ndown_up, "sp_ndown_up");
  sc_trace(trace_file, cpu->io_registers->sp_en, "sp_en");
  sc_trace(trace_file, cpu->io_registers->rampz_out, "rampz_out");
#endif

#ifdef SIGNAL_ALU_AVR
  // ALU_AVR
  sc_trace(trace_file, cpu->alu->alu_data_r_in, "alu_data_r_in");
  sc_trace(trace_file, cpu->alu->alu_data_d_in, "alu_data_d_in");
  sc_trace(trace_file, cpu->alu->alu_c_flag_in, "alu_c_flag_in");
  sc_trace(trace_file, cpu->alu->alu_z_flag_in, "alu_z_flag_in");

  sc_trace(trace_file, cpu->alu->idc_add, "idc_add");
  sc_trace(trace_file, cpu->alu->idc_adc, "idc_adc");
  sc_trace(trace_file, cpu->alu->idc_adiw, "idc_adiw");
  sc_trace(trace_file, cpu->alu->idc_sub, "idc_sub");
  sc_trace(trace_file, cpu->alu->idc_subi, "idc_subi");
  sc_trace(trace_file, cpu->alu->idc_sbc, "idc_sbc");
  sc_trace(trace_file, cpu->alu->idc_sbci, "idc_sbci");
  sc_trace(trace_file, cpu->alu->idc_sbiw, "idc_sbiw");
  sc_trace(trace_file, cpu->alu->adiw_st, "adiw_st");
  sc_trace(trace_file, cpu->alu->sbiw_st, "sbiw_st");
  sc_trace(trace_file, cpu->alu->idc_and, "idc_and");
  sc_trace(trace_file, cpu->alu->idc_andi, "idc_andi");
  sc_trace(trace_file, cpu->alu->idc_or, "idc_or");
  sc_trace(trace_file, cpu->alu->idc_ori, "idc_ori");
  sc_trace(trace_file, cpu->alu->idc_eor, "idc_eor");
  sc_trace(trace_file, cpu->alu->idc_com, "idc_com");
  sc_trace(trace_file, cpu->alu->idc_neg, "idc_neg");
  sc_trace(trace_file, cpu->alu->idc_inc, "idc_inc");
  sc_trace(trace_file, cpu->alu->idc_dec, "idc_dec");
  sc_trace(trace_file, cpu->alu->idc_cp, "idc_cp");
  sc_trace(trace_file, cpu->alu->idc_cpc, "idc_cpc");
  sc_trace(trace_file, cpu->alu->idc_cpi, "idc_cpi");
  sc_trace(trace_file, cpu->alu->idc_cpse, "idc_cpse");
  sc_trace(trace_file, cpu->alu->idc_lsr, "idc_lsr");
  sc_trace(trace_file, cpu->alu->idc_ror, "idc_ror");
  sc_trace(trace_file, cpu->alu->idc_asr, "idc_asr");
  sc_trace(trace_file, cpu->alu->idc_swap, "idc_swap");
  sc_trace(trace_file, cpu->alu->alu_data_out, "alu_data_out");

  sc_trace(trace_file, cpu->alu->alu_c_flag_out, "alu_c_flag_out");
  sc_trace(trace_file, cpu->alu->alu_z_flag_out, "alu_z_flag_out");
  sc_trace(trace_file, cpu->alu->alu_n_flag_out, "alu_n_flag_out");
  sc_trace(trace_file, cpu->alu->alu_v_flag_out, "alu_v_flag_out");
  sc_trace(trace_file, cpu->alu->alu_s_flag_out, "alu_s_flag_out");
  sc_trace(trace_file, cpu->alu->alu_h_flag_out, "alu_h_flag_out");
#endif

#ifdef SIGNAL_TIMER_COUNTER
  // timer_counter
  sc_trace(trace_file, irq_io->tc->clk, "clk");
  sc_trace(trace_file, irq_io->tc->reset, "reset");

  sc_trace(trace_file, irq_io->tc->adr, "adr");
  sc_trace(trace_file, irq_io->tc->dbus_in, "dbus_in");
  sc_trace(trace_file, irq_io->tc->dbus_out, "dbus_out");
  sc_trace(trace_file, irq_io->tc->iore, "iore");
  sc_trace(trace_file, irq_io->tc->iowe, "iowe");
  sc_trace(trace_file, irq_io->tc->out_en, "out_en");
  sc_trace(trace_file, irq_io->tc->ext1, "ext1");
  sc_trace(trace_file, irq_io->tc->ext2, "ext2");
  sc_trace(trace_file, irq_io->tc->tosc1, "tosc1");
  sc_trace(trace_file, irq_io->tc->oc0_pwm0, "oc0_pwm0");
  sc_trace(trace_file, irq_io->tc->oc1a_pwm1a, "oc1a_pwm1a");
  sc_trace(trace_file, irq_io->tc->oc1b_pwm1b, "oc1b_pwm1b");
  sc_trace(trace_file, irq_io->tc->oc2_pwm2, "oc2_pwm2");
 
  sc_trace(trace_file, irq_io->tc->tc0ovfirq, "tc0ovfirq");
  sc_trace(trace_file, irq_io->tc->tc0ovfirq_ack, "tc0ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq, "tc0cmpirq");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq_ack, "tc0cmpirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq, "tc2ovfirq");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq_ack, "tc2ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq, "tc2cmpirq");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq_ack, "tc2cmpirq_ack");  
  sc_trace(trace_file, irq_io->tc->tc1ovfirq, "tc1ovfirq");
  sc_trace(trace_file, irq_io->tc->tc1ovfirq_ack, "tc1ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq, "tc1cmpairq");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq_ack, "tc1cmpairq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq, "tc1cmpbirq");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq_ack, "tc1cmpbirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1icirq, "tc1icirq");
  sc_trace(trace_file, irq_io->tc->tc1icirq_ack, "tc1icirq_ack");
#endif

#ifdef SIGNAL_INTERRUPT
  //
  // Signals involving interrupt requests
  //
  // All connections to/from timer_counter
  sc_trace(trace_file, irq_io->tc->out_en, "timer_counter.out_en");
  sc_trace(trace_file, irq_io->tc->ext1, "timer_counter.ext1");
  sc_trace(trace_file, irq_io->tc->ext2, "timer_counter.ext2");
  sc_trace(trace_file, irq_io->tc->tosc1, "timer_counter.tosc1");
  sc_trace(trace_file, irq_io->tc->oc0_pwm0, "timer_counter.oc0_pwm0");
  sc_trace(trace_file, irq_io->tc->oc1a_pwm1a, "timer_counter.oc1a_pwm1a");
  sc_trace(trace_file, irq_io->tc->oc1b_pwm1b, "timer_counter.oc1b_pwm1b");
  sc_trace(trace_file, irq_io->tc->oc2_pwm2, "timer_counter.oc2_pwm2");
  sc_trace(trace_file, irq_io->tc->tc0ovfirq, "timer_counter.tc0ovfirq");
  sc_trace(trace_file, irq_io->tc->tc0ovfirq_ack, "timer_counter.tc0ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq, "timer_counter.tc0cmpirq");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq_ack, "timer_counter.tc0cmpirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq, "timer_counter.tc2ovfirq");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq_ack, "timer_counter.tc2ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq, "timer_counter.tc2cmpirq");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq_ack, "timer_counter.tc2cmpirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1ovfirq, "timer_counter.tc1ovfirq");
  sc_trace(trace_file, irq_io->tc->tc1ovfirq_ack, "timer_counter.tc1ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq, "timer_counter.tc1cmpairq");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq_ack, "timer_counter.tc1cmpairq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq, "timer_counter.tc1cmpbirq");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq_ack, "timer_counter.tc1cmpbirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1icirq, "timer_counter.tc1icirq");
  sc_trace(trace_file, irq_io->tc->tc1icirq_ack, "timer_counter.tc1icirq_ack");

  // registers in timer_counter
  sc_trace(trace_file, irq_io->tc->tccr0, "timer_counter._tccr0");
  sc_trace(trace_file, irq_io->tc->assr, "timer_counter._assr");
  sc_trace(trace_file, irq_io->tc->timsk, "timer_counter._timsk");
  sc_trace(trace_file, irq_io->tc->tifr, "timer_counter._tifr");
  sc_trace(trace_file, irq_io->tc->tcnt0, "timer_counter._tcnt0");
  sc_trace(trace_file, irq_io->tc->ocr0, "timer_counter._ocr0");
  sc_trace(trace_file, irq_io->tc->pre0cnt, "timer_counter._pre0cnt");

  sc_trace(trace_file, irq_io->tc->pck08, "timer_counter._pck08");
  sc_trace(trace_file, irq_io->tc->pck032, "timer_counter._pck032");
  sc_trace(trace_file, irq_io->tc->pck064, "timer_counter._pck064");
  sc_trace(trace_file, irq_io->tc->pck0128, "timer_counter._pck0128");
  sc_trace(trace_file, irq_io->tc->pck0256, "timer_counter._pck0256");
  sc_trace(trace_file, irq_io->tc->pck01024, "timer_counter._pck01024");
  sc_trace(trace_file, irq_io->tc->pck0_del, "timer_counter._pck0_del");
  sc_trace(trace_file, irq_io->tc->tcnt0_en, "timer_counter._tcnt0_en");
  sc_trace(trace_file, irq_io->tc->tcnt0_en2, "timer_counter._tcnt0_en2");
  sc_trace(trace_file, irq_io->tc->tcnt0_cmp_out, "timer_counter._tcnt0_cmp_out");
#endif

#ifdef SIGNAL_SLEEPHALT
  // Sleep/halt signals
  sc_trace(trace_file, irqok, "sleep_logic.irqok");
  sc_trace(trace_file, globint, "sleep_logic.globint");
  sc_trace(trace_file, cpu_clk, "sleep_logic.cpu_clk");

  sc_trace(trace_file, irq_io->hc->halt, "halt_cpu.halt");
  sc_trace(trace_file, irq_io->hc->cpuwait_i, "halt_cpu.cpuwait_i");
  sc_trace(trace_file, irq_io->hc->pcwe, "halt_cpu.pcwe");
  sc_trace(trace_file, irq_io->hc->pc_in, "halt_cpu.pc_in");
  sc_trace(trace_file, irq_io->hc->pc_out, "halt_cpu.pc_out");
  // sc_trace(trace_file, cpu->alu->irq_st1, "cpu.maincore.irq_st1");

  sc_trace(trace_file, irq_io->hc->regadr, "halt_cpu.regadr");
  sc_trace(trace_file, irq_io->hc->regre, "halt_cpu.regre");
  sc_trace(trace_file, irq_io->hc->regwe, "halt_cpu.regwe");
  sc_trace(trace_file, irq_io->hc->reg_in, "halt_cpu.reg_in");
  sc_trace(trace_file, irq_io->hc->reg_out, "halt_cpu.reg_out");
#endif

  /////////////////////////////////////////////////////////////////
  // Start the simulation
  /////////////////////////////////////////////////////////////////

  cout << "--->Start<---" << endl;
  sc_report::suppress_warnings(true);

  int runtime;
  if (argc == 3)
    runtime = atoi(argv[2]);
  else
    runtime = -1;

  sc_start(runtime);

  // Close trace file...
  sc_close_vcd_trace_file(trace_file);

  return 0;
}
