//
// $Id: main_signals.cc,v 1.1 2003/10/16 17:54:23 nj Exp $
//
#include <systemc.h>
#include <stdlib.h>
#include "config.h"
#include "constants.h"
#include "oneshot.h"
#include "avr_core.h"
#include "irq_io_module.h"
#include "instmemory.h"
#include "dataram.h"

int sc_main(int argc, char *argv[])
{
  sc_clock clk("clock", 20, SC_NS);
  sc_signal<bool> reset;

  if (argc == 1)
    {
      cout << "Usage:" << endl;
      cout << argv[0] << " <bin-file> [runlength (ns)]" << endl;
      return 0;
    }

  /////////////////////////////////////////////////////////////////
  // Signals to/from avr_core
  /////////////////////////////////////////////////////////////////
  sc_signal<bool> cpu_clk;
  sc_signal<sc_logic> cpuwait;
  // sc_signal<sc_logic> xx_cpuwait;
  sc_signal<sc_uint<16> > pc;
  sc_signal<sc_lv<16> > inst;
  sc_signal<sc_uint<6> > adr;
  sc_signal<sc_logic> iore;
  sc_signal<sc_logic> iowe;
  sc_signal<sc_uint<16> > ramadr;
  sc_signal<sc_logic> ramre;
  sc_signal<sc_logic> ramwe;
  sc_signal<sc_lv<8> > dbusin;
  sc_signal<sc_lv<8> > dbusout;

  sc_signal<sc_logic> sleepi;
  sc_signal<sc_logic> irqok;
  sc_signal<sc_logic> globint;

  sc_signal<sc_lv<23> > irqlines;
  sc_signal<sc_logic> irqack;
  sc_signal<sc_lv<5> > irqackad;

  // halt_signals
  sc_signal<sc_logic> pcwe;
  sc_signal<sc_uint<16> > pc_in;
  sc_signal<sc_uint<16> > pc_out;

  sc_signal<sc_uint<5> > regadr;
  sc_signal<sc_logic> regre;
  sc_signal<sc_logic> regwe;
  sc_signal<sc_lv<8> > reg_in;
  sc_signal<sc_lv<8> > reg_out;

  // Input to timer_counter
  sc_signal<sc_logic> out_en;

  sc_signal<sc_logic> ext1;
  sc_signal<sc_logic> ext2;

  // All signals in irqlines
  sc_signal<sc_logic> tosc1;
  sc_signal<sc_logic> oc0_pwm0;
  sc_signal<sc_logic> oc1a_pwm1a;
  sc_signal<sc_logic> oc1b_pwm1b;
  sc_signal<sc_logic> oc2_pwm2;

  sc_signal<sc_logic> tc1ovfirq;
  sc_signal<sc_logic> tc1cmpairq;
  sc_signal<sc_logic> tc1cmpbirq;
  sc_signal<sc_logic> tc1icirq;

  sc_signal<sc_lv<8> > dbusout_ram;


  /////////////////////////////////////////////////////////////////
  // Init components
  /////////////////////////////////////////////////////////////////
#ifdef WRITE_INIT
  cout << "INIT oneshot (oneshot1)" << endl;
#endif
  oneshot *oneshot1;
  oneshot1 = new oneshot("reset-blok");
  oneshot1->clk(clk.signal());
  oneshot1->reset(reset);

#ifdef WRITE_INIT
  cout << "INIT avr_core (cpu)" << endl;
#endif
  avr_core *cpu;
  cpu = new avr_core("avr_core");
  cpu->cp2(cpu_clk);
  cpu->ireset(reset);
  cpu->cpuwait(cpuwait);
  cpu->pc(pc);
  cpu->inst(inst);
  cpu->adr(adr);
  cpu->iore(iore);
  cpu->iowe(iowe);
  cpu->ramadr(ramadr);
  cpu->ramre(ramre);
  cpu->ramwe(ramwe);
  cpu->dbusin(dbusin);
  cpu->dbusout(dbusout);
  cpu->sleepi(sleepi);
  cpu->irqok(irqok);
  cpu->globint(globint); // unused...
  cpu->irqlines(irqlines);
  cpu->irqack(irqack);
  cpu->irqackad(irqackad);
  cpu->pcwe(pcwe);
  cpu->pc_in(pc_in);
  cpu->pc_out(pc_out);
  cpu->regadr(regadr);
  cpu->regre(regre);
  cpu->regwe(regwe);
  cpu->reg_in(reg_in);
  cpu->reg_out(reg_out);

#ifdef WRITE_INIT
  cout << "INIT irq_io_modulec (irq_io)" << endl;
#endif
  irq_io_module *irq_io;
  irq_io = new irq_io_module("irq_io_module");
  irq_io->clk(clk.signal());
  irq_io->reset(reset);
  irq_io->cpuwait(cpuwait);
  irq_io->cpu_clk(cpu_clk);
  irq_io->irqack(irqack);
  irq_io->irqackad(irqackad);
  irq_io->irqlines(irqlines);
  irq_io->ramre(ramre);
  irq_io->adr(adr);
  irq_io->iore(iore);
  irq_io->iowe(iowe);
  irq_io->dbusin(dbusin);
  irq_io->dbusout(dbusout);
  irq_io->sleepi(sleepi);
  // irq_io->irqok(irqok);
  irq_io->globint(globint);
  irq_io->dbusout_ram(dbusout_ram);

  irq_io->pcwe(pcwe);
  irq_io->pc_in(pc_in);
  irq_io->pc_out(pc_out);

  irq_io->regadr(regadr);
  irq_io->regre(regre);
  irq_io->regwe(regwe);
  irq_io->reg_in(reg_in);
  irq_io->reg_out(reg_out);

#ifdef WRITE_INIT
  cout << "INIT instmemory (imem)" << endl;
#endif
  instmemory *imem;
  imem = new instmemory("imemory", argv[1]);
  imem->clk(clk.signal());
  imem->reset(reset);
  imem->pc(pc);
  imem->inst(inst);

#ifdef WRITE_INIT
  cout << "INIT dataram (dmem)" << endl;
#endif
  dataram *dmem;
  dmem = new dataram("dmemory", argv[1]);
  dmem->clk(clk.signal());
  dmem->ramadr(ramadr);
  dmem->ramre(ramre);
  dmem->ramwe(ramwe);
  dmem->dbusin(dbusout); // NB! dbusout from avr_core
  dmem->dbusout(dbusout_ram);


  /////////////////////////////////////////////////////////////////
  // Set constant values
  /////////////////////////////////////////////////////////////////
  // cpuwait  = SC_LOGIC_0;

  /*
  x_irqlines[0] = SC_LOGIC_0;
  x_irqlines[1] = SC_LOGIC_0;
  x_irqlines[2] = SC_LOGIC_0;
  x_irqlines[3] = SC_LOGIC_0;
  x_irqlines[4] = SC_LOGIC_0;
  x_irqlines[5] = SC_LOGIC_0;
  x_irqlines[6] = SC_LOGIC_0;
  x_irqlines[7] = SC_LOGIC_0;
  x_irqlines[10] = SC_LOGIC_0;
  x_irqlines[11] = SC_LOGIC_0;
  x_irqlines[12] = SC_LOGIC_0;
  x_irqlines[13] = SC_LOGIC_0;
  x_irqlines[14] = SC_LOGIC_0;
  x_irqlines[17] = SC_LOGIC_0;
  x_irqlines[18] = SC_LOGIC_0;

  // to timer_counter inputs
  ext1   = SC_LOGIC_0;
  ext2   = SC_LOGIC_0;
  tosc1  = SC_LOGIC_0;

  // No external interrupts
  ext_int_in = "00000000";
  */

  /////////////////////////////////////////////////////////////////
  // Trace file - VCD format...
  /////////////////////////////////////////////////////////////////

  sc_trace_file *trace_file;
  trace_file = sc_create_vcd_trace_file("main.trace");

#ifdef SIGNAL_OTHER
  // General signals
  sc_trace(trace_file, clk.signal(), "clk");
  sc_trace(trace_file, reset, "reset");

  // Connections to avr_core
  sc_trace(trace_file, cpuwait, "cpuwait");
  sc_trace(trace_file, pc, "pc");
  sc_trace(trace_file, inst, "inst");
  sc_trace(trace_file, adr, "adr");
  sc_trace(trace_file, iore, "iore");
  sc_trace(trace_file, iowe, "iowe");
  sc_trace(trace_file, ramadr, "ramadr");
  sc_trace(trace_file, ramre, "ramre");
  sc_trace(trace_file, ramwe, "ramwe");
  sc_trace(trace_file, dbusin, "dbusin");
  sc_trace(trace_file, dbusout, "dbusout");
  sc_trace(trace_file, irqlines, "irqlines");
  sc_trace(trace_file, irqack, "irqack");
  sc_trace(trace_file, irqackad, "irqackad");
#endif

#ifdef SIGNAL_AVR_CORE
  // All connections to/from AVR_CORE
  sc_trace(trace_file, cpu->sg_dbusin, "cpu.sg_dbusin");
  sc_trace(trace_file, cpu->sg_dbusout, "cpu.sg_dbusout");
  sc_trace(trace_file, cpu->sg_adr, "cpu.sg_adr");
  sc_trace(trace_file, cpu->sg_iowe, "cpu.sg_iowe");
  sc_trace(trace_file, cpu->sg_iore, "cpu.sg_iore");

  sc_trace(trace_file, cpu->sg_idc_adc, "cpu.sg_idc_adc");
  sc_trace(trace_file, cpu->sg_idc_add, "cpu.sg_idc_add");
  sc_trace(trace_file, cpu->sg_idc_adiw, "cpu.sg_idc_adiw");
  sc_trace(trace_file, cpu->sg_idc_sub, "cpu.sg_idc_sub");
  sc_trace(trace_file, cpu->sg_idc_subi, "cpu.sg_idc_subi");
  sc_trace(trace_file, cpu->sg_idc_sbc, "cpu.sg_idc_sbc");
  sc_trace(trace_file, cpu->sg_idc_sbci, "cpu.sg_idc_sbci");
  sc_trace(trace_file, cpu->sg_idc_sbiw, "cpu.sg_idc_sbiw");
  sc_trace(trace_file, cpu->sg_adiw_st, "cpu.sg_adiw_st");
  sc_trace(trace_file, cpu->sg_sbiw_st, "cpu.sg_sbiw_st");
  sc_trace(trace_file, cpu->sg_idc_and, "cpu.sg_idc_and");
  sc_trace(trace_file, cpu->sg_idc_andi, "cpu.sg_idc_andi");
  sc_trace(trace_file, cpu->sg_idc_or, "cpu.sg_idc_or");
  sc_trace(trace_file, cpu->sg_idc_ori, "cpu.sg_idc_ori");
  sc_trace(trace_file, cpu->sg_idc_eor, "cpu.sg_idc_eor");
  sc_trace(trace_file, cpu->sg_idc_com, "cpu.sg_idc_com");
  sc_trace(trace_file, cpu->sg_idc_neg, "cpu.sg_idc_neg");
  sc_trace(trace_file, cpu->sg_idc_inc, "cpu.sg_idc_inc");
  sc_trace(trace_file, cpu->sg_idc_dec, "cpu.sg_idc_dec");
  sc_trace(trace_file, cpu->sg_idc_cp, "cpu.sg_idc_cp");
  sc_trace(trace_file, cpu->sg_idc_cpc, "cpu.sg_idc_cpc");
  sc_trace(trace_file, cpu->sg_idc_cpi, "cpu.sg_idc_cpi");
  sc_trace(trace_file, cpu->sg_idc_cpse, "cpu.sg_idc_cpse");
  sc_trace(trace_file, cpu->sg_idc_lsr, "cpu.sg_idc_lsr");
  sc_trace(trace_file, cpu->sg_idc_ror, "cpu.sg_idc_ror");
  sc_trace(trace_file, cpu->sg_idc_asr, "cpu.sg_idc_asr");
  sc_trace(trace_file, cpu->sg_idc_swap, "cpu.sg_idc_swap");
  sc_trace(trace_file, cpu->sg_idc_sbi, "cpu.sg_idc_sbi");
  // sc_trace(trace_file, cpu->sg_idc_st, "cpu.sg_idc_st");
  sc_trace(trace_file, cpu->sg_idc_bld, "cpu.sg_idc_bld");
  sc_trace(trace_file, cpu->sg_idc_bst, "cpu.sg_idc_bst");
  sc_trace(trace_file, cpu->sg_idc_bset, "cpu.sg_idc_bset");
  sc_trace(trace_file, cpu->sg_idc_bclr, "cpu.sg_idc_bclr");
  sc_trace(trace_file, cpu->sg_idc_sbic, "cpu.sg_idc_sbic");
  sc_trace(trace_file, cpu->sg_idc_sbis, "cpu.sg_idc_sbis");
  sc_trace(trace_file, cpu->sg_idc_sbrs, "cpu.sg_idc_sbrs");
  sc_trace(trace_file, cpu->sg_idc_sbrc, "cpu.sg_idc_sbrc");
  sc_trace(trace_file, cpu->sg_idc_brbs, "cpu.sg_idc_brbs");
  sc_trace(trace_file, cpu->sg_idc_brbc, "cpu.sg_idc_brbc");
  sc_trace(trace_file, cpu->sg_idc_reti, "cpu.sg_idc_reti");

  sc_trace(trace_file, cpu->sg_alu_data_r_in, "cpu.sg_alu_data_r_in");
  sc_trace(trace_file, cpu->sg_alu_data_d_in, "cpu.sg_alu_data_d_in");
  sc_trace(trace_file, cpu->sg_alu_data_out, "cpu.sg_alu_data_out");
  sc_trace(trace_file, cpu->sg_reg_rd_in, "cpu.sg_reg_rd_in");
  sc_trace(trace_file, cpu->sg_reg_rd_out, "cpu.sg_reg_rd_out");
  sc_trace(trace_file, cpu->sg_reg_rr_out, "cpu.sg_reg_rr_out");
  sc_trace(trace_file, cpu->sg_reg_rd_adr, "cpu.sg_reg_rd_adr");
  sc_trace(trace_file, cpu->sg_reg_rr_adr, "cpu.sg_reg_rr_adr");
  sc_trace(trace_file, cpu->sg_reg_h_out, "cpu.sg_reg_h_out");
  sc_trace(trace_file, cpu->sg_reg_z_out, "cpu.sg_reg_z_out");
  sc_trace(trace_file, cpu->sg_reg_h_adr, "cpu.sg_reg_h_adr");
  sc_trace(trace_file, cpu->sg_reg_rd_wr, "cpu.sg_reg_rd_wr");
  sc_trace(trace_file, cpu->sg_post_inc, "cpu.sg_post_inc");
  sc_trace(trace_file, cpu->sg_pre_dec, "cpu.sg_pre_dec");
  sc_trace(trace_file, cpu->sg_reg_h_wr, "cpu.sg_reg_h_wr");
  sc_trace(trace_file, cpu->sg_sreg_fl_in, "cpu.sg_sreg_fl_in");
  sc_trace(trace_file, cpu->sg_sreg_out, "cpu.sg_sreg_out");
  sc_trace(trace_file, cpu->sg_sreg_fl_wr_en, "cpu.sg_sreg_fl_wr_en");
  sc_trace(trace_file, cpu->sg_spl_out, "cpu.sg_spl_out");
  sc_trace(trace_file, cpu->sg_sph_out, "cpu.sg_sph_out");
  sc_trace(trace_file, cpu->sg_rampz_out, "cpu.sg_rampz_out");
  sc_trace(trace_file, cpu->sg_sp_ndown_up, "cpu.sg_sp_ndown_up");
  sc_trace(trace_file, cpu->sg_sp_en, "cpu.sg_sp_en");

  sc_trace(trace_file, cpu->sg_bit_test_op_out, "cpu.sg_bit_test_op_out");
  sc_trace(trace_file, cpu->sg_alu_c_flag_out, "cpu.sg_alu_c_flag_out");
  sc_trace(trace_file, cpu->sg_alu_z_flag_out, "cpu.sg_alu_z_flag_out");
  sc_trace(trace_file, cpu->sg_alu_n_flag_out, "cpu.sg_alu_n_flag_out");
  sc_trace(trace_file, cpu->sg_alu_v_flag_out, "cpu.sg_alu_v_flag_out");
  sc_trace(trace_file, cpu->sg_alu_s_flag_out, "cpu.sg_alu_s_flag_out");
  sc_trace(trace_file, cpu->sg_alu_h_flag_out, "cpu.sg_alu_h_flag_out");

  sc_trace(trace_file, cpu->sg_sleepi, "cpu.sg_sleepi");
#endif


#ifdef SIGNAL_PM_FETCH_DEC
  sc_trace(trace_file, cpu->maincore->ramadr_reg_in, "cpu.maincore.ramadr_reg_in");
  sc_trace(trace_file, cpu->maincore->ramadr_reg_en, "cpu.maincore.ramadr_reg_en");
  sc_trace(trace_file, cpu->maincore->irqack_int, "cpu.maincore.irqack_int");
  sc_trace(trace_file, cpu->maincore->irqackad_int, "cpu.maincore.irqackad_int");
  sc_trace(trace_file, cpu->maincore->two_word_inst, "cpu.maincore.two_word_inst");
  sc_trace(trace_file, cpu->maincore->ram_adr_int, "cpu.maincore.ram_adr_int");
  sc_trace(trace_file, cpu->maincore->adiw_sbiw_encoder_out, "cpu.maincore.adiw_sbiw_encoder_out");
  sc_trace(trace_file, cpu->maincore->adiw_sbiw_encoder_mux_out, "cpu.maincore.adiw_sbiw_encoder_mux_out");

  sc_trace(trace_file, cpu->maincore->program_counter_tmp, "cpu.maincore.program_counter_tmp");
  sc_trace(trace_file, cpu->maincore->program_counter, "cpu.maincore.program_counter");
  sc_trace(trace_file, cpu->maincore->program_counter_in, "cpu.maincore.program_counter_in");
  sc_trace(trace_file, cpu->maincore->program_counter_high_fr, "cpu.maincore.program_counter_high_fr");
  sc_trace(trace_file, cpu->maincore->program_counter_low_fr, "cpu.maincore.program_counter_low_fr");
  sc_trace(trace_file, cpu->maincore->pc_low, "cpu.maincore.pc_low");
  sc_trace(trace_file, cpu->maincore->pc_high, "cpu.maincore.pc_high");
  sc_trace(trace_file, cpu->maincore->pc_low_en, "cpu.maincore.pc_low_en");
  sc_trace(trace_file, cpu->maincore->pc_high_en, "cpu.maincore.pc_high_en");

  sc_trace(trace_file, cpu->maincore->offset_brbx, "cpu.maincore.offset_brbx");
  sc_trace(trace_file, cpu->maincore->offset_rxx, "cpu.maincore.offset_rxx");
  sc_trace(trace_file, cpu->maincore->pa15_pm, "cpu.maincore.pa15_pm");
  sc_trace(trace_file, cpu->maincore->alu_reg_wr, "cpu.maincore.alu_reg_wr");
  sc_trace(trace_file, cpu->maincore->reg_file_adr_space, "cpu.maincore.reg_file_adr_space");
  sc_trace(trace_file, cpu->maincore->io_file_adr_space, "cpu.maincore.io_file_adr_space");
  sc_trace(trace_file, cpu->maincore->irq_start, "cpu.maincore.irq_start");

  sc_trace(trace_file, cpu->maincore->cpu_busy, "cpu.maincore.cpu_busy");
  sc_trace(trace_file, cpu->maincore->instruction_reg, "cpu.maincore.instruction_reg");
  sc_trace(trace_file, cpu->maincore->instruction_code_reg, "cpu.maincore.instruction_code_reg");
  sc_trace(trace_file, cpu->maincore->instruction_reg_ena, "cpu.maincore.instruction_reg_ena");
  sc_trace(trace_file, cpu->maincore->irq_vector_adr, "cpu.maincore.irq_vector_adr");
  sc_trace(trace_file, cpu->maincore->irq_pr_encoder, "cpu.maincore.irq_pr_encoder");
  sc_trace(trace_file, cpu->maincore->pc_for_interrupt, "cpu.maincore.pc_for_interrupt");
  sc_trace(trace_file, cpu->maincore->gp_reg_tmp, "cpu.maincore.gp_reg_tmp");
  sc_trace(trace_file, cpu->maincore->cbi_sbi_io_adr_tmp, "cpu.maincore.cbi_sbi_io_adr_tmp");
  sc_trace(trace_file, cpu->maincore->cbi_sbi_bit_num_tmp, "cpu.maincore.cbi_sbi_bit_num_tmp");

  sc_trace(trace_file, cpu->maincore->idc_adc, "cpu.maincore.idc_adc");
  sc_trace(trace_file, cpu->maincore->idc_add, "cpu.maincore.idc_add");
  sc_trace(trace_file, cpu->maincore->idc_adiw, "cpu.maincore.idc_adiw");
  sc_trace(trace_file, cpu->maincore->idc_and, "cpu.maincore.idc_and");
  sc_trace(trace_file, cpu->maincore->idc_andi, "cpu.maincore.idc_andi");
  sc_trace(trace_file, cpu->maincore->idc_asr, "cpu.maincore.idc_asr");
  sc_trace(trace_file, cpu->maincore->idc_bclr, "cpu.maincore.idc_bclr");
  sc_trace(trace_file, cpu->maincore->idc_bld, "cpu.maincore.idc_bld");
  sc_trace(trace_file, cpu->maincore->idc_brbc, "cpu.maincore.idc_brbc");
  sc_trace(trace_file, cpu->maincore->idc_brbs, "cpu.maincore.idc_brbs");
  sc_trace(trace_file, cpu->maincore->idc_bset, "cpu.maincore.idc_bset");
  sc_trace(trace_file, cpu->maincore->idc_bst, "cpu.maincore.idc_bst");
  sc_trace(trace_file, cpu->maincore->idc_call, "cpu.maincore.idc_call");
  sc_trace(trace_file, cpu->maincore->idc_cbi, "cpu.maincore.idc_cbi");
  sc_trace(trace_file, cpu->maincore->idc_com, "cpu.maincore.idc_com");
  sc_trace(trace_file, cpu->maincore->idc_cp, "cpu.maincore.idc_cp");
  sc_trace(trace_file, cpu->maincore->idc_cpc, "cpu.maincore.idc_cpc");
  sc_trace(trace_file, cpu->maincore->idc_cpi, "cpu.maincore.idc_cpi");
  sc_trace(trace_file, cpu->maincore->idc_cpse, "cpu.maincore.idc_cpse");
  sc_trace(trace_file, cpu->maincore->idc_dec, "cpu.maincore.idc_dec");
  sc_trace(trace_file, cpu->maincore->idc_elpm, "cpu.maincore.idc_elpm");
  sc_trace(trace_file, cpu->maincore->idc_eor, "cpu.maincore.idc_eor");
  sc_trace(trace_file, cpu->maincore->idc_icall, "cpu.maincore.idc_icall");
  sc_trace(trace_file, cpu->maincore->idc_ijmp, "cpu.maincore.idc_ijmp");
  sc_trace(trace_file, cpu->maincore->idc_in, "cpu.maincore.idc_in");
  sc_trace(trace_file, cpu->maincore->idc_inc, "cpu.maincore.idc_inc");
  sc_trace(trace_file, cpu->maincore->idc_jmp, "cpu.maincore.idc_jmp");
  sc_trace(trace_file, cpu->maincore->idc_ld_x, "cpu.maincore.idc_ld_x");
  sc_trace(trace_file, cpu->maincore->idc_ld_y, "cpu.maincore.idc_ld_y");
  sc_trace(trace_file, cpu->maincore->idc_ldd_y, "cpu.maincore.idc_ldd_y");
  sc_trace(trace_file, cpu->maincore->idc_ld_z, "cpu.maincore.idc_ld_z");
  sc_trace(trace_file, cpu->maincore->idc_ldd_z, "cpu.maincore.idc_ldd_z");
  sc_trace(trace_file, cpu->maincore->idc_ldi, "cpu.maincore.idc_ldi");
  sc_trace(trace_file, cpu->maincore->idc_lds, "cpu.maincore.idc_lds");
  sc_trace(trace_file, cpu->maincore->idc_lpm, "cpu.maincore.idc_lmp");
  sc_trace(trace_file, cpu->maincore->idc_lsr, "cpu.maincore.idc_lsr");
  sc_trace(trace_file, cpu->maincore->idc_mov, "cpu.maincore.idc_mov");
  sc_trace(trace_file, cpu->maincore->idc_mul, "cpu.maincore.idc_mul");
  sc_trace(trace_file, cpu->maincore->idc_neg, "cpu.maincore.idc_neg");
  sc_trace(trace_file, cpu->maincore->idc_nop, "cpu.maincore.idc_nop");
  sc_trace(trace_file, cpu->maincore->idc_or, "cpu.maincore.idc_or");
  sc_trace(trace_file, cpu->maincore->idc_ori, "cpu.maincore.idc_ori");
  sc_trace(trace_file, cpu->maincore->idc_out, "cpu.maincore.idc_out");
  sc_trace(trace_file, cpu->maincore->idc_pop, "cpu.maincore.idc_pop");
  sc_trace(trace_file, cpu->maincore->idc_push, "cpu.maincore.idc_push");
  sc_trace(trace_file, cpu->maincore->idc_rcall, "cpu.maincore.idc_rcall");
  sc_trace(trace_file, cpu->maincore->idc_ret, "cpu.maincore.idc_ret");
  sc_trace(trace_file, cpu->maincore->idc_reti, "cpu.maincore.idc_reti");
  sc_trace(trace_file, cpu->maincore->idc_rjmp, "cpu.maincore.idc_rjmp");
  sc_trace(trace_file, cpu->maincore->idc_ror, "cpu.maincore.idc_ror");
  sc_trace(trace_file, cpu->maincore->idc_sbc, "cpu.maincore.idc_sbc");
  sc_trace(trace_file, cpu->maincore->idc_sbci, "cpu.maincore.idc_sbci");
  sc_trace(trace_file, cpu->maincore->idc_sbi, "cpu.maincore.idc_sbi");
  sc_trace(trace_file, cpu->maincore->idc_sbic, "cpu.maincore.idc_sbic");
  sc_trace(trace_file, cpu->maincore->idc_sbis, "cpu.maincore.idc_sbis");
  sc_trace(trace_file, cpu->maincore->idc_sbiw, "cpu.maincore.idc_sbiw");
  sc_trace(trace_file, cpu->maincore->adiw_st, "cpu.maincore.adiw_st");
  sc_trace(trace_file, cpu->maincore->sbiw_st, "cpu.maincore.sbiw_st");
  sc_trace(trace_file, cpu->maincore->idc_sbrc, "cpu.maincore.idc_sbrc");
  sc_trace(trace_file, cpu->maincore->idc_sbrs, "cpu.maincore.idc_sbrs");
  sc_trace(trace_file, cpu->maincore->idc_sleep, "cpu.maincore.idc_sleep");
  sc_trace(trace_file, cpu->maincore->idc_st_x, "cpu.maincore.idc_st_x");
  sc_trace(trace_file, cpu->maincore->idc_st_y, "cpu.maincore.idc_st_y");
  sc_trace(trace_file, cpu->maincore->idc_std_y, "cpu.maincore.idc_std_y");
  sc_trace(trace_file, cpu->maincore->idc_st_z, "cpu.maincore.idc_st_z");
  sc_trace(trace_file, cpu->maincore->idc_std_z, "cpu.maincore.idc_std_z");
  sc_trace(trace_file, cpu->maincore->idc_sts, "cpu.maincore.idc_sts");
  sc_trace(trace_file, cpu->maincore->idc_sub, "cpu.maincore.idc_sub");
  sc_trace(trace_file, cpu->maincore->idc_subi, "cpu.maincore.idc_subi");
  sc_trace(trace_file, cpu->maincore->idc_swap, "cpu.maincore.idc_swap");
  sc_trace(trace_file, cpu->maincore->idc_wdr, "cpu.maincore.idc_wdr");
  sc_trace(trace_file, cpu->maincore->idc_psinc, "cpu.maincore.idc_psinc");
  sc_trace(trace_file, cpu->maincore->idc_prdec, "cpu.maincore.idc_prdec");

  sc_trace(trace_file, cpu->maincore->sreg_bop_wr_en, "cpu.maincore.sreg_bop_wr_en");
  sc_trace(trace_file, cpu->maincore->sreg_adr_eq, "cpu.maincore.sreg_adr_eq");
  sc_trace(trace_file, cpu->maincore->x_reg_rd_adr, "cpu.maincore.x_reg_rd_adr");
#endif


#ifdef SIGNAL_BIT_PROCESSOR
  sc_trace(trace_file, cpu->bit_proc->sreg_t_flag, "cpu.bit_processor.sreg_t_flag");
  sc_trace(trace_file, cpu->bit_proc->temp_in_data, "cpu.bit_processor.temp_in_data");
  sc_trace(trace_file, cpu->bit_proc->sreg_t_temp, "cpu.bit_processor.sreg_t_temp");
  sc_trace(trace_file, cpu->bit_proc->bit_num_decode, "cpu.bit_processor.bit_num_decode");
  sc_trace(trace_file, cpu->bit_proc->bit_pr_sreg_out_int, "cpu.bit_processor.bit_pr_sreg_out_int");
  sc_trace(trace_file, cpu->bit_proc->bit_test_in, "cpu.bit_processor.bit_test_in");
  sc_trace(trace_file, cpu->bit_proc->bit_test_mux_out, "cpu.bit_processor.bit_test_mux_out");
  sc_trace(trace_file, cpu->bit_proc->branch_decode, "cpu.bit_processor.branch_decode");
  sc_trace(trace_file, cpu->bit_proc->branch_mux, "cpu.bit_processor.branch_mux");
#endif


#ifdef SIGNAL_ALU_AVR
  sc_trace(trace_file, cpu->alu->adder_nadd_sub, "cpu.alu_avr.adder_nadd_sub");
  sc_trace(trace_file, cpu->alu->adder_v_flag_out, "cpu.alu_avr.adder_v_flag_out");
  sc_trace(trace_file, cpu->alu->adder_carry, "cpu.alu_avr.adder_carry");
  sc_trace(trace_file, cpu->alu->adder_d_in, "cpu.alu_avr.adder_d_in");
  sc_trace(trace_file, cpu->alu->adder_r_in, "cpu.alu_avr.adder_r_in");
  sc_trace(trace_file, cpu->alu->adder_out, "cpu.alu_avr.adder_out");

  sc_trace(trace_file, cpu->alu->neg_op_in, "cpu.alu_avr.neg_op_in");
  sc_trace(trace_file, cpu->alu->neg_op_carry, "cpu.alu_avr.neg_op_carry");
  sc_trace(trace_file, cpu->alu->neg_op_out, "cpu.alu_avr.neg_op_out");

  sc_trace(trace_file, cpu->alu->incdec_op_in, "cpu.alu_avr.incdec_op_in");
  sc_trace(trace_file, cpu->alu->incdec_op_carry, "cpu.alu_avr.incdec_op_carry");
  sc_trace(trace_file, cpu->alu->incdec_op_out, "cpu.alu_avr.incdec_op_out");

  sc_trace(trace_file, cpu->alu->com_op_out, "cpu.alu_avr.com_op_out");
  sc_trace(trace_file, cpu->alu->and_op_out, "cpu.alu_avr.and_op_out");
  sc_trace(trace_file, cpu->alu->or_op_out, "cpu.alu_avr.or_op_out");
  sc_trace(trace_file, cpu->alu->eor_op_out, "cpu.alu_avr.eor_op_out");

  sc_trace(trace_file, cpu->alu->right_shift_out, "cpu.alu_avr.right_shift_out");
  sc_trace(trace_file, cpu->alu->swap_out, "cpu.alu_avr.swap_out");
#endif


#ifdef SIGNAL_REG_FILE
  sc_trace(trace_file, cpu->gp_register_file->r26h, "cpu.regfile.r26h");
  sc_trace(trace_file, cpu->gp_register_file->r27h, "cpu.regfile.r27h");
  sc_trace(trace_file, cpu->gp_register_file->r28h, "cpu.regfile.r28h");
  sc_trace(trace_file, cpu->gp_register_file->r29h, "cpu.regfile.r29h");
  sc_trace(trace_file, cpu->gp_register_file->r30h, "cpu.regfile.r30h");
  sc_trace(trace_file, cpu->gp_register_file->r31h, "cpu.regfile.r31h");

  sc_trace(trace_file, cpu->gp_register_file->sg_adr16_postinc, "cpu.regfile.sg_adr16_postinc");
  sc_trace(trace_file, cpu->gp_register_file->sg_adr16_predec, "cpu.regfile.sg_adr16_predec");
  sc_trace(trace_file, cpu->gp_register_file->reg_h_in, "cpu.regfile.reg_h_in");
  sc_trace(trace_file, cpu->gp_register_file->sg_tmp_h_data, "cpu.regfile.sg_tmp_h_data");

  sc_trace(trace_file, cpu->gp_register_file->register_file[0], "cpu.regfile.register_file[0]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[1], "cpu.regfile.register_file[1]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[2], "cpu.regfile.register_file[2]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[3], "cpu.regfile.register_file[3]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[4], "cpu.regfile.register_file[4]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[5], "cpu.regfile.register_file[5]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[6], "cpu.regfile.register_file[6]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[7], "cpu.regfile.register_file[7]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[8], "cpu.regfile.register_file[8]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[9], "cpu.regfile.register_file[9]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[10], "cpu.regfile.register_file[10]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[11], "cpu.regfile.register_file[11]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[12], "cpu.regfile.register_file[12]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[13], "cpu.regfile.register_file[13]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[14], "cpu.regfile.register_file[14]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[15], "cpu.regfile.register_file[15]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[16], "cpu.regfile.register_file[16]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[17], "cpu.regfile.register_file[17]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[18], "cpu.regfile.register_file[18]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[19], "cpu.regfile.register_file[19]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[20], "cpu.regfile.register_file[20]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[21], "cpu.regfile.register_file[21]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[22], "cpu.regfile.register_file[22]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[23], "cpu.regfile.register_file[23]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[24], "cpu.regfile.register_file[24]");
  sc_trace(trace_file, cpu->gp_register_file->register_file[25], "cpu.regfile.register_file[25]");
#endif


#ifdef SIGNAL_IO_REG_FILE
  sc_trace(trace_file, cpu->io_registers->sreg, "cpu.io_reg_file.sreg");
  sc_trace(trace_file, cpu->io_registers->sph, "cpu.io_reg_file.sph");
  sc_trace(trace_file, cpu->io_registers->spl, "cpu.io_reg_file.spl");
  sc_trace(trace_file, cpu->io_registers->rampz, "cpu.io_reg_file.rampz");
  sc_trace(trace_file, cpu->io_registers->sp_int, "cpu.io_reg_file.sp_int");
  sc_trace(trace_file, cpu->io_registers->sp_intp, "cpu.io_reg_file.sp_intp");
  sc_trace(trace_file, cpu->io_registers->sp_intm, "cpu.io_reg_file.sp_intm");
  sc_trace(trace_file, cpu->io_registers->sp_res, "cpu.io_reg_file.sp_res");
#endif


#ifdef SIGNAL_INTERRUPT
  //
  // Signals involving interrupt requests
  //
  // All connections to/from timer_counter
  sc_trace(trace_file, irq_io->tc->out_en, "timer_counter.out_en");
  sc_trace(trace_file, irq_io->tc->ext1, "timer_counter.ext1");
  sc_trace(trace_file, irq_io->tc->ext2, "timer_counter.ext2");
  sc_trace(trace_file, irq_io->tc->tosc1, "timer_counter.tosc1");
  sc_trace(trace_file, irq_io->tc->oc0_pwm0, "timer_counter.oc0_pwm0");
  sc_trace(trace_file, irq_io->tc->oc1a_pwm1a, "timer_counter.oc1a_pwm1a");
  sc_trace(trace_file, irq_io->tc->oc1b_pwm1b, "timer_counter.oc1b_pwm1b");
  sc_trace(trace_file, irq_io->tc->oc2_pwm2, "timer_counter.oc2_pwm2");
  sc_trace(trace_file, irq_io->tc->tc0ovfirq, "timer_counter.tc0ovfirq");
  sc_trace(trace_file, irq_io->tc->tc0ovfirq_ack, "timer_counter.tc0ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq, "timer_counter.tc0cmpirq");
  sc_trace(trace_file, irq_io->tc->tc0cmpirq_ack, "timer_counter.tc0cmpirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq, "timer_counter.tc2ovfirq");
  sc_trace(trace_file, irq_io->tc->tc2ovfirq_ack, "timer_counter.tc2ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq, "timer_counter.tc2cmpirq");
  sc_trace(trace_file, irq_io->tc->tc2cmpirq_ack, "timer_counter.tc2cmpirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1ovfirq, "timer_counter.tc1ovfirq");
  sc_trace(trace_file, irq_io->tc->tc1ovfirq_ack, "timer_counter.tc1ovfirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq, "timer_counter.tc1cmpairq");
  sc_trace(trace_file, irq_io->tc->tc1cmpairq_ack, "timer_counter.tc1cmpairq_ack");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq, "timer_counter.tc1cmpbirq");
  sc_trace(trace_file, irq_io->tc->tc1cmpbirq_ack, "timer_counter.tc1cmpbirq_ack");
  sc_trace(trace_file, irq_io->tc->tc1icirq, "timer_counter.tc1icirq");
  sc_trace(trace_file, irq_io->tc->tc1icirq_ack, "timer_counter.tc1icirq_ack");

  // registers in timer_counter
  sc_trace(trace_file, irq_io->tc->tccr0, "timer_counter._tccr0");
  sc_trace(trace_file, irq_io->tc->assr, "timer_counter._assr");
  sc_trace(trace_file, irq_io->tc->timsk, "timer_counter._timsk");
  sc_trace(trace_file, irq_io->tc->tifr, "timer_counter._tifr");
  sc_trace(trace_file, irq_io->tc->tcnt0, "timer_counter._tcnt0");
  sc_trace(trace_file, irq_io->tc->ocr0, "timer_counter._ocr0");
  sc_trace(trace_file, irq_io->tc->pre0cnt, "timer_counter._pre0cnt");

  sc_trace(trace_file, irq_io->tc->pck08, "timer_counter._pck08");
  sc_trace(trace_file, irq_io->tc->pck032, "timer_counter._pck032");
  sc_trace(trace_file, irq_io->tc->pck064, "timer_counter._pck064");
  sc_trace(trace_file, irq_io->tc->pck0128, "timer_counter._pck0128");
  sc_trace(trace_file, irq_io->tc->pck0256, "timer_counter._pck0256");
  sc_trace(trace_file, irq_io->tc->pck01024, "timer_counter._pck01024");
  sc_trace(trace_file, irq_io->tc->pck0_del, "timer_counter._pck0_del");
  sc_trace(trace_file, irq_io->tc->tcnt0_en, "timer_counter._tcnt0_en");
  sc_trace(trace_file, irq_io->tc->tcnt0_en2, "timer_counter._tcnt0_en2");
  sc_trace(trace_file, irq_io->tc->tcnt0_cmp_out, "timer_counter._tcnt0_cmp_out");
#endif

#ifdef SIGNAL_SLEEPHALT
  // Sleep/halt signals
  sc_trace(trace_file, irqok, "sleep_logic.irqok");
  sc_trace(trace_file, globint, "sleep_logic.globint");
  sc_trace(trace_file, cpu_clk, "sleep_logic.cpu_clk");

  sc_trace(trace_file, irq_io->hc->halt, "halt_cpu.halt");
  sc_trace(trace_file, irq_io->hc->cpuwait_i, "halt_cpu.cpuwait_i");
  sc_trace(trace_file, irq_io->hc->pcwe, "halt_cpu.pcwe");
  sc_trace(trace_file, irq_io->hc->pc_in, "halt_cpu.pc_in");
  sc_trace(trace_file, irq_io->hc->pc_out, "halt_cpu.pc_out");
  // sc_trace(trace_file, cpu->maincore->irq_st1, "cpu.maincore.irq_st1");

  sc_trace(trace_file, irq_io->hc->regadr, "halt_cpu.regadr");
  sc_trace(trace_file, irq_io->hc->regre, "halt_cpu.regre");
  sc_trace(trace_file, irq_io->hc->regwe, "halt_cpu.regwe");
  sc_trace(trace_file, irq_io->hc->reg_in, "halt_cpu.reg_in");
  sc_trace(trace_file, irq_io->hc->reg_out, "halt_cpu.reg_out");
#endif

#ifdef SIGNAL_ALU_INPUTS_ONLY 
  sc_trace(trace_file, cpu->alu->alu_data_r_in, "alu_data_r_in");
  sc_trace(trace_file, cpu->alu->alu_data_d_in, "alu_data_d_in");
  sc_trace(trace_file, cpu->alu->alu_c_flag_in, "alu_c_flag_in");
  sc_trace(trace_file, cpu->alu->alu_z_flag_in, "alu_z_flag_in");
  sc_trace(trace_file, cpu->alu->idc_add, "idc_add");
  sc_trace(trace_file, cpu->alu->idc_adc, "idc_adc");
  sc_trace(trace_file, cpu->alu->idc_adiw, "idc_adiw");
  sc_trace(trace_file, cpu->alu->idc_sub, "idc_sub");
  sc_trace(trace_file, cpu->alu->idc_subi, "idc_subi");
  sc_trace(trace_file, cpu->alu->idc_sbc, "idc_sbc");
  sc_trace(trace_file, cpu->alu->idc_sbci, "idc_sbci");
  sc_trace(trace_file, cpu->alu->idc_sbiw, "idc_sbiw");

  sc_trace(trace_file, cpu->alu->adiw_st, "adiw_st");
  sc_trace(trace_file, cpu->alu->sbiw_st, "sbiw_st");

  sc_trace(trace_file, cpu->alu->idc_and, "idc_and");
  sc_trace(trace_file, cpu->alu->idc_andi, "idc_andi");
  sc_trace(trace_file, cpu->alu->idc_or, "idc_or");
  sc_trace(trace_file, cpu->alu->idc_ori, "idc_ori");
  sc_trace(trace_file, cpu->alu->idc_eor, "idc_eor");
  sc_trace(trace_file, cpu->alu->idc_com, "idc_com");
  sc_trace(trace_file, cpu->alu->idc_neg, "idc_neg");

  sc_trace(trace_file, cpu->alu->idc_inc, "idc_inc");
  sc_trace(trace_file, cpu->alu->idc_dec, "idc_dec");

  sc_trace(trace_file, cpu->alu->idc_cp, "idc_cp");
  sc_trace(trace_file, cpu->alu->idc_cpc, "idc_cpc");
  sc_trace(trace_file, cpu->alu->idc_cpi, "idc_cpi");
  sc_trace(trace_file, cpu->alu->idc_cpse, "idc_cpse");
  sc_trace(trace_file, cpu->alu->idc_lsr, "idc_lsr");
  sc_trace(trace_file, cpu->alu->idc_ror, "idc_ror");
  sc_trace(trace_file, cpu->alu->idc_asr, "idc_asr");
  sc_trace(trace_file, cpu->alu->idc_swap, "idc_swap");
#endif

#ifdef SIGNAL_PM_FETCH_DEC_INPUTS_ONLY
  sc_trace(trace_file, cpu->maincore->clk, "clk");
  sc_trace(trace_file, cpu->maincore->nrst, "nrst");
  sc_trace(trace_file, cpu->maincore->inst, "inst");
  sc_trace(trace_file, cpu->maincore->dbusin, "dbusin");
  sc_trace(trace_file, cpu->maincore->irqlines, "irqlines");
  sc_trace(trace_file, cpu->maincore->alu_data_out, "alu_data_out");
  sc_trace(trace_file, cpu->maincore->alu_c_flag_out, "alu_c_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_z_flag_out, "alu_z_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_n_flag_out, "alu_n_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_v_flag_out, "alu_v_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_s_flag_out, "alu_s_flag_out");
  sc_trace(trace_file, cpu->maincore->alu_h_flag_out, "alu_h_flag_out");
  sc_trace(trace_file, cpu->maincore->reg_rd_out, "reg_rd_out");
  sc_trace(trace_file, cpu->maincore->reg_rr_out, "reg_rr_out");
  sc_trace(trace_file, cpu->maincore->reg_h_out, "reg_h_out");
  sc_trace(trace_file, cpu->maincore->reg_z_out, "reg_z_out");
  sc_trace(trace_file, cpu->maincore->sreg_out, "sreg_out");
  sc_trace(trace_file, cpu->maincore->spl_out, "spl_out");
  sc_trace(trace_file, cpu->maincore->sph_out, "sph_out");
  sc_trace(trace_file, cpu->maincore->rampz_out, "rampz_out");
  sc_trace(trace_file, cpu->maincore->bitpr_io_out, "bitpr_io_out");
  sc_trace(trace_file, cpu->maincore->bit_pr_sreg_out, "bit_pr_sreg_out");
  sc_trace(trace_file, cpu->maincore->bld_op_out, "bld_op_out");
  sc_trace(trace_file, cpu->maincore->bit_test_op_out, "bit_test_op_out");
  sc_trace(trace_file, cpu->maincore->pcwe, "pcwe");
  sc_trace(trace_file, cpu->maincore->pc_in, "pc_in");
  sc_trace(trace_file, cpu->maincore->pc_out, "pc_out");
#endif

#ifdef SIGNAL_AVR_CORE_INPUTS_ONLY
  sc_trace(trace_file, clk.signal(), "cp2");
  sc_trace(trace_file, reset, "ireset");
  sc_trace(trace_file, cpuwait, "cpuwait");
  sc_trace(trace_file, inst, "inst");
  sc_trace(trace_file, dbusin, "dbusin");
  sc_trace(trace_file, irqlines, "irqlines");
  sc_trace(trace_file, regadr, "regadr");
  sc_trace(trace_file, regre, "regre");
  sc_trace(trace_file, regwe, "regwe");
  sc_trace(trace_file, reg_out, "reg_out");
  sc_trace(trace_file, cpu->io_registers->rampz, "avr_core/io_registers/rampz_reg");
#endif


  /////////////////////////////////////////////////////////////////
  // Start the simulation
  /////////////////////////////////////////////////////////////////

  cout << "--->Start<---" << endl;
  sc_report::suppress_warnings(true);

  int runtime;
  if (argc == 3)
    runtime = atoi(argv[2]);
  else
    runtime = -1;

  sc_start(runtime);

  // Close trace file...
  sc_close_vcd_trace_file(trace_file);

  return 0;
}
