//
// $Id: service_module.cc,v 1.5 2003/07/30 16:45:31 nj Exp $
//
#include "service_module.h"

//
//
//
void service_module::externalintsynchronizer()
{
  if (!ireset)
    {
      ext_int_resync = "00000000";
    }
  else
    {
      ext_int_resync = ext_int_in;
    }
}

//
//
//
void service_module::externalintreg()
{
  if (!ireset)
    {
      ext_int_inreg = "00000000";
    }
  else
    {
      ext_int_inreg = ext_int_resync.read();
    }
}

//
// Set main control unit register
//
void service_module::mcucr_reg()
{
  sc_lv<8> v_mcucr = mcucr;
  sc_lv<8> v_dbus_in = dbus_in.read();

  if (!ireset)
    {
      v_mcucr = "00000000";
    }
  else
    {
      if ((mcucr_sel.read() == 1) & (iowe.read() == 1))
	{
	  v_mcucr[7] = v_dbus_in[7];
	  v_mcucr[6] = v_dbus_in[6];
	  v_mcucr[5] = v_dbus_in[5];
	  v_mcucr[4] = v_dbus_in[4];
	  v_mcucr[3] = v_dbus_in[3];
	}
    }
  sleep_en = v_mcucr[5];
  esram_ws = v_mcucr[6];
  esram_en = v_mcucr[7];
  mcucr = v_mcucr;
}

//
//
//
void service_module::eimsk_reg()
{
  if (!ireset)
    {
      eimsk = "00000000";
    }
  else
    {
      if ((eimsk_sel.read() == 1) & (iowe.read() == 1))
	{
	  eimsk = dbus_in;
	}
    }
}

//
//
//
void service_module::eicr_reg()
{
  if (!ireset)
    {
      eicr = "00000000";
    }
  else
    {
      if ((eicr_sel.read() == 1) & (iowe.read() == 1))
	{
	  eicr = dbus_in;
	}
    }
}

//
//
//
void service_module::mcusr_reg()
{
  sc_lv<8> v_mcusr = mcusr;
  sc_lv<8> v_dbus_in = dbus_in;

  if (!ireset)
    {
      v_mcusr = "00000000";
    }
  else
    {
      if ((mcusr_sel.read() == 1) & (iowe.read() == 1))
	{
	  v_mcusr[1] = v_dbus_in[1];
	  v_mcusr[0] = v_dbus_in[0];
	}
    }
  mcusr = v_mcusr;
}

//
//
//
#ifdef CI_ML_XDIV
void service_module::xdiv_reg()
{
  if (!ireset)
    {
      xdiv = "00000000";
    }
  else
    {
      if ((xdiv_sel.read() == 1) & (iowe.read() == 1))
	{
	  xdiv = dbus_in;
	}
    }
}
#endif

//
// Set EIFR register
//
void service_module::eifr_reg()
{
  sc_lv<4> v_ext_int_del; // = ext_int_del;
  sc_lv<8> v_eifr = eifr;
  sc_lv<4> v_edge_det_out = edge_det_out;
  sc_lv<8> v_dbus_in = dbus_in;
  sc_lv<4> v_extint_irq_ack = extint_irq_ack.read();
  sc_lv<8> v_ext_int_inreg = ext_int_inreg;

  if (!ireset)
    {
      eifr = "00000000";
      v_ext_int_del = "0000";
    }
  else
    {
      EIFR_GENERATE(3);
      EIFR_GENERATE(2);
      EIFR_GENERATE(1);
      EIFR_GENERATE(0);
      v_ext_int_del = v_ext_int_inreg.range(3,0);
    }
  eifr = v_eifr;
  ext_int_del = v_ext_int_del;
}

//
// Set signals named *_sel
//
// sensitive << adr << mcucr_sel << eimsk_sel << eifr_sel << eicr_sel << mcusr_sel << xdiv_sel << iore
void service_module::set_selsignals()
{
  sc_lv<6> ladr = adr.read();
  
  // Default values
  mcucr_sel = SC_LOGIC_0;
  eimsk_sel = SC_LOGIC_0;
  eifr_sel  = SC_LOGIC_0;
  eicr_sel  = SC_LOGIC_0;
  mcusr_sel = SC_LOGIC_0;
  xdiv_sel  = SC_LOGIC_0;
 
  if (ladr == MCUCR_Address) mcucr_sel = SC_LOGIC_1; // 53
  if (ladr == EIMSK_Address) eimsk_sel = SC_LOGIC_1; // 57
  if (ladr == EIFR_Address)  eifr_sel  = SC_LOGIC_1; // 56
  if (ladr == EICR_Address)  eicr_sel  = SC_LOGIC_1; // 58
  if (ladr == MCUSR_Address) mcusr_sel = SC_LOGIC_1; // 52
  if (ladr == XDIV_Address)  xdiv_sel  = SC_LOGIC_1; // 60

  out_en = (mcucr_sel | eimsk_sel | eifr_sel | eicr_sel | mcusr_sel | xdiv_sel) & iore.read();
}

//
// Edge detection
//
// sensitive << ext_int_inreg << ext_int_del << eicr
void service_module::edge_detector_logic()
{
  sc_lv<4> v_edge_det_out;
  sc_lv<8> v_ext_int_inreg = ext_int_inreg;
  sc_lv<4> v_ext_int_del = ext_int_del;
  sc_lv<8> v_eicr = eicr;

  EDGE_DET_GENERATE(3);
  EDGE_DET_GENERATE(2);
  EDGE_DET_GENERATE(1);
  EDGE_DET_GENERATE(0);

  edge_det_out = v_edge_det_out;
}

//
// Update IRQ signals
//
// sensitive << extint_irq << eicr << eifr << eimsk << ext_int_inreg
void service_module::high_irqs()
{
  sc_lv<8> v_extint_irq = extint_irq;
  sc_lv<8> v_eicr = eicr;
  sc_lv<8> v_eifr = eifr;
  sc_lv<8> v_eimsk = eimsk;
  sc_lv<8> v_ext_int_inreg = ext_int_inreg;

  // First, low IRQs...
  v_extint_irq[3] = ~v_ext_int_inreg[3] & v_eimsk[3];
  v_extint_irq[2] = ~v_ext_int_inreg[2] & v_eimsk[2];
  v_extint_irq[1] = ~v_ext_int_inreg[1] & v_eimsk[1];
  v_extint_irq[0] = ~v_ext_int_inreg[0] & v_eimsk[0];

  // High, IRQs
  EXTINT_GENERATE(3);
  EXTINT_GENERATE(2);
  EXTINT_GENERATE(1);
  EXTINT_GENERATE(0);

  extint_irq = v_extint_irq;
}

//
// Output mux
//
// sensitive << mcucr << eimsk << eifr << eicr << mcusr << xdiv
// sensitive << xdiv_sel << mcusr_sel << eicr_sel << eifr_sel << eimsk_sel << mcucr_sel
void service_module::output_data_mux()
{
  sc_lv<8> v_mcucr = mcucr;
  sc_lv<8> v_eimsk = eimsk;
  sc_lv<8> v_eifr = eifr;
  sc_lv<8> v_eicr = eicr;
  sc_lv<8> v_mcusr = mcusr;
  sc_lv<8> v_xdiv = xdiv;
  sc_lv<8> v_dbus_out;

  OUTPUT_DATA_GENERATE(0);
  OUTPUT_DATA_GENERATE(1);
  OUTPUT_DATA_GENERATE(2);
  OUTPUT_DATA_GENERATE(3);
  OUTPUT_DATA_GENERATE(4);
  OUTPUT_DATA_GENERATE(5);
  OUTPUT_DATA_GENERATE(6);
  OUTPUT_DATA_GENERATE(7);

  dbus_out = v_dbus_out;
}
